@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __ACTIONS_H__
@@ -98,18 +98,36 @@ void               action_message          (GimpDisplay          *display,
   if (! drawable) \
     return
 
+#define return_if_no_drawables(image,drawables,data) \
+  return_if_no_image (image,data); \
+  drawables = gimp_image_get_selected_drawables (image); \
+  if (! drawables) \
+    return
+
 #define return_if_no_layer(image,layer,data) \
   return_if_no_image (image,data); \
   layer = gimp_image_get_active_layer (image); \
   if (! layer) \
     return
 
+#define return_if_no_layers(image,layers,data) \
+  return_if_no_image (image,data); \
+  layers = gimp_image_get_selected_layers (image); \
+  if (! layers) \
+    return
+
 #define return_if_no_channel(image,channel,data) \
   return_if_no_image (image,data); \
   channel = gimp_image_get_active_channel (image); \
   if (! channel) \
     return
 
+#define return_if_no_channels(image,channels,data) \
+  return_if_no_image (image,data); \
+  channels = gimp_image_get_selected_channels (image); \
+  if (! channels) \
+    return
+
 #define return_if_no_vectors(image,vectors,data) \
   return_if_no_image (image,data); \
   vectors = gimp_image_get_active_vectors (image); \
