@@ -4,9 +4,9 @@
  * GimpConfig object property dumper.
  * Copyright (C) 2001-2006  Sven Neumann <sven@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -24,20 +23,21 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#include <cairo.h>
+#include <gegl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <glib-object.h>
+#ifdef G_OS_WIN32
+#include <io.h> /* get_osfhandle */
+#include <gio/gwin32outputstream.h>
+#else
+#include <gio/gunixoutputstream.h>
+#endif
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpconfig/gimpconfig.h"
 
-#ifdef G_OS_WIN32
-#include "libgimpbase/gimpwin32-io.h"
-#endif
-
 #include "config-types.h"
 
 #include "gimpconfig-dump.h"
@@ -47,23 +47,36 @@
 
 static void    dump_gimprc_system   (GimpConfig       *rc,
                                      GimpConfigWriter *writer,
-                                     gint              fd);
+                                     GOutputStream    *output);
 static void    dump_gimprc_manpage  (GimpConfig       *rc,
                                      GimpConfigWriter *writer,
-                                     gint              fd);
+                                     GOutputStream    *output);
 static gchar * dump_describe_param  (GParamSpec       *param_spec);
-static void    dump_with_linebreaks (gint              fd,
+static void    dump_with_linebreaks (GOutputStream    *output,
                                      const gchar      *text);
 
 
 gboolean
-gimp_config_dump (GimpConfigDumpFormat  format)
+gimp_config_dump (GObject              *gimp,
+                  GimpConfigDumpFormat  format)
 {
+  GOutputStream    *output;
   GimpConfigWriter *writer;
   GimpConfig       *rc;
 
-  rc = g_object_new (GIMP_TYPE_RC, NULL);
-  writer = gimp_config_writer_new_fd (1);
+  g_return_val_if_fail (G_IS_OBJECT (gimp), FALSE);
+
+  rc = g_object_new (GIMP_TYPE_RC,
+                     "gimp", gimp,
+                     NULL);
+
+#ifdef G_OS_WIN32
+  output = g_win32_output_stream_new ((gpointer) _get_osfhandle (fileno (stdout)), FALSE);
+#else
+  output = g_unix_output_stream_new (1, FALSE);
+#endif
+
+  writer = gimp_config_writer_new_from_stream (output, NULL, NULL);
 
   switch (format)
     {
@@ -79,15 +92,16 @@ gimp_config_dump (GimpConfigDumpFormat  format)
       break;
 
     case GIMP_CONFIG_DUMP_GIMPRC_SYSTEM:
-      dump_gimprc_system (rc, writer, 1);
+      dump_gimprc_system (rc, writer, output);
       break;
 
     case GIMP_CONFIG_DUMP_GIMPRC_MANPAGE:
-      dump_gimprc_manpage (rc, writer, 1);
+      dump_gimprc_manpage (rc, writer, output);
       break;
     }
 
   gimp_config_writer_finish (writer, NULL, NULL);
+  g_object_unref (output);
   g_object_unref (rc);
 
   return TRUE;
@@ -105,14 +119,14 @@ static const gchar system_gimprc_header[] =
 "documents the default values and shows what changes are possible.\n"
 "\n"
 "The variable ${gimp_dir} is set to the value of the environment "
-"variable GIMP2_DIRECTORY or, if that is not set, the compiled-in "
-"default value is used.  If GIMP2_DIRECTORY is not an absolute path, "
+"variable GIMP3_DIRECTORY or, if that is not set, the compiled-in "
+"default value is used.  If GIMP3_DIRECTORY is not an absolute path, "
 "it is interpreted relative to your home directory.";
 
 static void
 dump_gimprc_system (GimpConfig       *rc,
                     GimpConfigWriter *writer,
-                    gint              fd)
+                    GOutputStream    *output)
 {
   GObjectClass  *klass;
   GParamSpec   **property_specs;
@@ -146,7 +160,40 @@ dump_gimprc_system (GimpConfig       *rc,
       gimp_config_writer_comment_mode (writer, TRUE);
       gimp_config_writer_linefeed (writer);
 
-      gimp_config_serialize_property (rc, prop_spec, writer);
+      if (! strcmp (prop_spec->name, "num-processors"))
+        {
+          gimp_config_writer_open (writer, "num-processors");
+          gimp_config_writer_printf (writer, "1");
+          gimp_config_writer_close (writer);
+        }
+      else if (! strcmp (prop_spec->name, "tile-cache-size"))
+        {
+          gimp_config_writer_open (writer, "tile-cache-size");
+          gimp_config_writer_printf (writer, "2g");
+          gimp_config_writer_close (writer);
+        }
+      else if (! strcmp (prop_spec->name, "undo-size"))
+        {
+          gimp_config_writer_open (writer, "undo-size");
+          gimp_config_writer_printf (writer, "1g");
+          gimp_config_writer_close (writer);
+        }
+      else if (! strcmp (prop_spec->name, "mypaint-brush-path"))
+        {
+          gchar *path = g_strdup_printf ("@mypaint_brushes_dir@%s"
+                                         "~/.mypaint/brushes",
+                                         G_SEARCHPATH_SEPARATOR_S);
+
+          gimp_config_writer_open (writer, "mypaint-brush-path");
+          gimp_config_writer_string (writer, path);
+          gimp_config_writer_close (writer);
+
+          g_free (path);
+        }
+      else
+        {
+          gimp_config_serialize_property (rc, prop_spec, writer);
+        }
 
       gimp_config_writer_comment_mode (writer, FALSE);
       gimp_config_writer_linefeed (writer);
@@ -159,7 +206,7 @@ dump_gimprc_system (GimpConfig       *rc,
 static const gchar man_page_header[] =
 ".\\\" This man-page is auto-generated by gimp --dump-gimprc-manpage.\n"
 "\n"
-".TH GIMPRC 5 \"Version @GIMP_VERSION@\" \"GIMP Manual Pages\"\n"
+".TH GIMPRC 5 \"Version " GIMP_VERSION "\" \"GIMP Manual Pages\"\n"
 ".SH NAME\n"
 "gimprc \\- gimp configuration file\n"
 ".SH DESCRIPTION\n"
@@ -167,7 +214,7 @@ static const gchar man_page_header[] =
 ".B gimprc\n"
 "file is a configuration file read by GIMP when it starts up.  There\n"
 "are two of these: one system-wide one stored in\n"
-"@gimpsysconfdir@/gimprc and a per-user \\fB$HOME\\fP/@gimpdir@/gimprc\n"
+"@gimpsysconfdir@/gimprc and a per-user @manpage_gimpdir@/gimprc\n"
 "which may override system settings.\n"
 "\n"
 "Comments are introduced by a hash sign (#), and continue until the end\n"
@@ -209,22 +256,30 @@ static const gchar *man_page_path =
 ".TP\n"
 ".I gimp_dir\n"
 "The personal gimp directory which is set to the value of the environment\n"
-"variable GIMP2_DIRECTORY or to ~/@gimpdir@.\n"
+"variable GIMP3_DIRECTORY or to @manpage_gimpdir@.\n"
 ".TP\n"
 ".I gimp_data_dir\n"
 "Base for paths to shareable data, which is set to the value of the\n"
-"environment variable GIMP2_DATADIR or to the compiled-in default value\n"
+"environment variable GIMP3_DATADIR or to the compiled-in default value\n"
 "@gimpdatadir@.\n"
 ".TP\n"
 ".I gimp_plug_in_dir\n"
-"Base to paths for architecture-specific plugins and modules, which is set\n"
-"to the value of the environment variable GIMP2_PLUGINDIR or to the\n"
+"Base to paths for architecture-specific plug-ins and modules, which is set\n"
+"to the value of the environment variable GIMP3_PLUGINDIR or to the\n"
 "compiled-in default value @gimpplugindir@.\n"
 ".TP\n"
 ".I gimp_sysconf_dir\n"
 "Path to configuration files, which is set to the value of the environment\n"
-"variable GIMP2_SYSCONFDIR or to the compiled-in default value \n"
+"variable GIMP3_SYSCONFDIR or to the compiled-in default value \n"
 "@gimpsysconfdir@.\n"
+".TP\n"
+".I gimp_cache_dir\n"
+"Path to cached files, which is set to the value of the environment\n"
+"variable GIMP3_CACHEDIR or to the system default for per-user cached files.\n"
+".TP\n"
+".I gimp_temp_dir\n"
+"Path to temporary files, which is set to the value of the environment\n"
+"variable GIMP3_TEMPDIR or to the system default for temporary files.\n"
 "\n";
 
 static const gchar man_page_footer[] =
@@ -233,26 +288,25 @@ static const gchar man_page_footer[] =
 ".I @gimpsysconfdir@/gimprc\n"
 "System-wide configuration file\n"
 ".TP\n"
-".I \\fB$HOME\\fP/@gimpdir@/gimprc\n"
+".I @manpage_gimpdir@/gimprc\n"
 "Per-user configuration file\n"
 "\n"
 ".SH \"SEE ALSO\"\n"
-".BR gimp (1),\n"
-".BR gimptool (1),\n"
-".BR gimp\\-remote (1)\n";
+".BR gimp (1)\n";
 
 
 static void
 dump_gimprc_manpage (GimpConfig       *rc,
                      GimpConfigWriter *writer,
-                     gint              fd)
+                     GOutputStream    *output)
 {
   GObjectClass  *klass;
   GParamSpec   **property_specs;
   guint          n_property_specs;
   guint          i;
 
-  write (fd, man_page_header, strlen (man_page_header));
+  g_output_stream_printf (output, NULL, NULL, NULL,
+                          "%s", man_page_header);
 
   klass = G_OBJECT_GET_CLASS (rc);
   property_specs = g_object_class_list_properties (klass, &n_property_specs);
@@ -261,6 +315,7 @@ dump_gimprc_manpage (GimpConfig       *rc,
     {
       GParamSpec *prop_spec = property_specs[i];
       gchar      *desc;
+      gboolean    success;
 
       if (! (prop_spec->flags & GIMP_CONFIG_PARAM_SERIALIZE))
         continue;
@@ -268,16 +323,63 @@ dump_gimprc_manpage (GimpConfig       *rc,
       if (prop_spec->flags & GIMP_CONFIG_PARAM_IGNORE)
         continue;
 
-      write (fd, ".TP\n", strlen (".TP\n"));
+      g_output_stream_printf (output, NULL, NULL, NULL,
+                              ".TP\n");
 
-      if (gimp_config_serialize_property (rc, prop_spec, writer))
+      if (! strcmp (prop_spec->name, "num-processors"))
         {
-          write (fd, "\n", 1);
+          gimp_config_writer_open (writer, "num-processors");
+          gimp_config_writer_printf (writer, "1");
+          gimp_config_writer_close (writer);
+
+          success = TRUE;
+        }
+      else if (! strcmp (prop_spec->name, "tile-cache-size"))
+        {
+          gimp_config_writer_open (writer, "tile-cache-size");
+          gimp_config_writer_printf (writer, "2g");
+          gimp_config_writer_close (writer);
+
+          success = TRUE;
+        }
+      else if (! strcmp (prop_spec->name, "undo-size"))
+        {
+          gimp_config_writer_open (writer, "undo-size");
+          gimp_config_writer_printf (writer, "1g");
+          gimp_config_writer_close (writer);
+
+          success = TRUE;
+        }
+      else if (! strcmp (prop_spec->name, "mypaint-brush-path"))
+        {
+          gchar *path = g_strdup_printf ("@mypaint_brushes_dir@%s"
+                                         "~/.mypaint/brushes",
+                                         G_SEARCHPATH_SEPARATOR_S);
+
+          gimp_config_writer_open (writer, "mypaint-brush-path");
+          gimp_config_writer_string (writer, path);
+          gimp_config_writer_close (writer);
+
+          g_free (path);
+
+          success = TRUE;
+        }
+      else
+        {
+          success = gimp_config_serialize_property (rc, prop_spec, writer);
+        }
+
+      if (success)
+        {
+          g_output_stream_printf (output, NULL, NULL, NULL,
+                                  "\n");
 
           desc = dump_describe_param (prop_spec);
 
-          dump_with_linebreaks (fd, desc);
-          write (fd, "\n", 1);
+          dump_with_linebreaks (output, desc);
+
+          g_output_stream_printf (output, NULL, NULL, NULL,
+                                  "\n");
 
           g_free (desc);
         }
@@ -285,8 +387,10 @@ dump_gimprc_manpage (GimpConfig       *rc,
 
   g_free (property_specs);
 
-  write (fd, man_page_path,   strlen (man_page_path));
-  write (fd, man_page_footer, strlen (man_page_footer));
+  g_output_stream_printf (output, NULL, NULL, NULL,
+                          "%s", man_page_path);
+  g_output_stream_printf (output, NULL, NULL, NULL,
+                          "%s", man_page_footer);
 }
 
 
@@ -307,6 +411,8 @@ static const gchar display_format_description[] =
 "%Cx expands to x if the image is clean, the empty string otherwise\n"
 "%B  expands to (modified) if the image is dirty, the empty string otherwise\n"
 "%A  expands to (clean) if the image is clean, the empty string otherwise\n"
+"%Nx expands to x if the image is export-dirty, the empty string otherwise\n"
+"%Ex expands to x if the image is export-clean, the empty string otherwise\n"
 "%l  the number of layers\n"
 "%L  the number of layers (long form)\n"
 "%m  memory used by the image\n"
@@ -316,8 +422,14 @@ static const gchar display_format_description[] =
 "%W  image width in real-world units\n"
 "%h  image height in pixels\n"
 "%H  image height in real-world units\n"
+"%M  the image size expressed in megapixels\n"
 "%u  unit symbol\n"
-"%U  unit abbreviation\n\n";
+"%U  unit abbreviation\n"
+"%x  the width of the active layer/channel in pixels\n"
+"%X  the width of the active layer/channel in real-world units\n"
+"%y  the height of the active layer/channel in pixels\n"
+"%Y  the height of the active layer/channel in real-world units\n"
+"%o  the name of the image's color profile\n\n";
 
 
 static gchar *
@@ -417,6 +529,7 @@ dump_describe_param (GParamSpec *param_spec)
         case G_TYPE_UINT:
         case G_TYPE_LONG:
         case G_TYPE_ULONG:
+        case G_TYPE_INT64:
           values = "This is an integer value.";
           break;
         case G_TYPE_FLOAT:
@@ -481,15 +594,18 @@ dump_describe_param (GParamSpec *param_spec)
     g_warning ("FIXME: Can't tell anything about a %s.",
                g_type_name (param_spec->value_type));
 
-  return g_strdup_printf ("%s  %s", blurb, values);
+  if (strcmp (blurb, "") == 0)
+    return g_strdup_printf ("%s", values);
+  else
+    return g_strdup_printf ("%s  %s", blurb, values);
 }
 
 
 #define LINE_LENGTH 78
 
 static void
-dump_with_linebreaks (gint         fd,
-                      const gchar *text)
+dump_with_linebreaks (GOutputStream *output,
+                      const gchar   *text)
 {
   gint len = strlen (text);
 
@@ -500,7 +616,8 @@ dump_with_linebreaks (gint         fd,
 
       /*  groff doesn't like lines to start with a single quote  */
       if (*text == '\'')
-        write (fd, "\\&", 2);  /*  this represents a zero width space  */
+        g_output_stream_printf (output, NULL, NULL, NULL,
+                                "\\&");  /*  a zero width space  */
 
       for (t = text, i = 0, space = 0;
            *t != '\n' && (i <= LINE_LENGTH || space == 0) && i < len;
@@ -513,11 +630,13 @@ dump_with_linebreaks (gint         fd,
       if (i > LINE_LENGTH && space && *t != '\n')
         i = space;
 
-      write (fd, text, i);
-      write (fd, "\n", 1);
+      g_output_stream_write_all (output, text, i, NULL, NULL, NULL);
+      g_output_stream_printf (output, NULL, NULL, NULL,
+                              "\n");
 
       if (*t == '\n')
-        write (fd, ".br\n", 4);
+        g_output_stream_printf (output, NULL, NULL, NULL,
+                                ".br\n");
 
       i++;
 
