@@ -3,9 +3,9 @@
  *
  * Copyright (C) 2003  Sven Neumann <sven@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_CONFIG_DUMP_H__
@@ -31,7 +30,8 @@ typedef enum
 } GimpConfigDumpFormat;
 
 
-gboolean  gimp_config_dump (GimpConfigDumpFormat  format);
+gboolean  gimp_config_dump (GObject              *gimp,
+                            GimpConfigDumpFormat  format);
 
 
 #endif  /* __GIMP_CONFIG_DUMP_H__ */
