@@ -33,15 +33,28 @@ _("Sets the dynamics search path.")
 _("Sets the canvas padding color used if the padding mode is set to " \
   "custom color.")
 
+#define CANVAS_PADDING_IN_SHOW_ALL_BLURB \
+_("Specifies whether to keep the canvas padding when \"View -> Show All\" " \
+  "is enabled.")
+
 #define CANVAS_PADDING_MODE_BLURB \
 _("Specifies how the area around the image should be drawn.")
 
+#define CHECK_UPDATES_BLURB \
+_("Check for availability of GIMP updates through background internet queries.")
+
+#define CHECK_UPDATE_TIMESTAMP_BLURB \
+_("Timestamp of the last update check.")
+
 #define COLOR_MANAGEMENT_BLURB \
 "Defines the color management behavior."
 
 #define COLOR_PROFILE_POLICY_BLURB \
 _("How to handle embedded color profiles when opening a file.")
 
+#define COLOR_PROFILE_PATH_BLURB \
+_("Sets the default folder path for all color profile file dialogs.")
+
 #define CURSOR_MODE_BLURB \
 _("Sets the type of mouse pointers to use.")
 
@@ -64,6 +77,9 @@ _("Context-dependent mouse pointers are helpful.  They are enabled by " \
 "Specify a default tool preset.  The tool preset is searched for in the " \
 "specified tool prests path."
 
+#define DEFAULT_SHOW_ALL_BLURB \
+_("Show full image content by default.")
+
 #define DEFAULT_DOT_FOR_DOT_BLURB \
 _("When enabled, this will ensure that each pixel of an image gets " \
   "mapped to a pixel on the screen.")
@@ -94,16 +110,16 @@ _("This is the distance in pixels where Guide and Grid snapping " \
   "activates.")
 
 #define SNAP_TO_GUIDES_BLURB \
-"Snap to guides by default in new image windows."
+_("Snap to guides by default in new image windows.")
 
 #define SNAP_TO_GRID_BLURB \
-"Snap to the grid by default in new image windows."
+_("Snap to the grid by default in new image windows.")
 
 #define SNAP_TO_CANVAS_BLURB \
-"Snap to the canvas edges by default in new image windows."
+_("Snap to the canvas edges by default in new image windows.")
 
 #define SNAP_TO_PATH_BLURB \
-"Snap to the active path by default in new image windows."
+_("Snap to the active path by default in new image windows.")
 
 #define DEFAULT_THRESHOLD_BLURB \
 _("Tools such as fuzzy-select and bucket fill find regions based on a " \
@@ -175,6 +191,9 @@ _("Sets the browser used by the help system.")
 #define FILTER_TOOL_MAX_RECENT_BLURB \
 _("How many recent settings to keep around in filter tools.")
 
+#define FILTER_TOOL_USE_LAST_SETTINGS_BLURB \
+_("Default to the last used settings in filter tools.")
+
 #define FILTER_TOOL_SHOW_COLOR_OPTIONS_BLURB \
 _("Show advanced color options in filter tools.")
 
@@ -190,7 +209,7 @@ _("Promote imported images to floating point precision. Does not apply " \
 
 #define IMPORT_PROMOTE_DITHER_BLURB \
 _("When promoting imported images to floating point precision, also add " \
-  "minimal noise in order do distribute color values a bit.")
+  "minimal noise in order to distribute color values a bit.")
 
 #define IMPORT_ADD_ALPHA_BLURB \
 _("Add an alpha channel to all layers of imported images.")
@@ -198,10 +217,54 @@ _("Add an alpha channel to all layers of imported images.")
 #define IMPORT_RAW_PLUG_IN_BLURB \
 _("Which plug-in to use for importing raw digital camera files.")
 
+#define EXPORT_FILE_TYPE_BLURB \
+_("Export file type used by default.")
+
+#define EXPORT_COLOR_PROFILE_BLURB \
+_("Export the image's color profile by default.")
+
+#define EXPORT_COMMENT_BLURB \
+_("Export the image's comment by default.")
+
+#define EXPORT_THUMBNAIL_BLURB \
+_("Export the image's thumbnail by default")
+
+/* Translators: tooltip for configuration option (checkbox).
+ * It determines how file export plug-ins handle Exif by default.
+ */
+#define EXPORT_METADATA_EXIF_BLURB \
+_("Export Exif metadata by default.")
+
+/* Translators: tooltip for configuration option (checkbox).
+ * It determines how file export plug-ins handle XMP by default.
+ */
+#define EXPORT_METADATA_XMP_BLURB \
+_("Export XMP metadata by default.")
+
+/* Translators: tooltip for configuration option (checkbox).
+ * It determines how file export plug-ins handle IPTC by default.
+ */
+#define EXPORT_METADATA_IPTC_BLURB \
+_("Export IPTC metadata by default.")
+
+#define GENERATE_BACKTRACE_BLURB \
+_("Try generating debug data for bug reporting when appropriate.")
+
+#define WIN32_POINTER_INPUT_API_BLURB \
+_("Sets the preferred pen and touch input API.")
+
 #define INITIAL_ZOOM_TO_FIT_BLURB \
 _("When enabled, this will ensure that the full image is visible after a " \
   "file is opened, otherwise it will be displayed with a scale of 1:1.")
 
+#define DRAG_ZOOM_MODE_BLURB \
+_("Whether to zoom based on distance moved or time spent moving, when " \
+  "zooming via dragging the mouse.")
+
+#define DRAG_ZOOM_SPEED_BLURB \
+_("Adjusts the rate at which dragging the mouse will zoom the canvas, " \
+  "in percentage.")
+
 #define INTERPOLATION_TYPE_BLURB \
 _("Sets the level of interpolation used for scaling and other " \
   "transformations.")
@@ -212,9 +275,21 @@ _("Sets the level of interpolation used for scaling and other " \
 #define LANGUAGE_BLURB \
 _("Specifies the language to use for the user interface.")
 
+#define LAST_KNOWN_RELEASE_BLURB \
+_("The last known release version of GIMP as queried from official website.")
+
+#define CONFIG_VERSION_BLURB \
+_("The version of GIMP config files.")
+
 #define LAST_OPENED_SIZE_BLURB \
 _("How many recently opened image filenames to keep on the File menu.")
 
+#define LAST_RELEASE_TIMESTAMP_BLURB \
+_("The timestamp for the last known release date.")
+
+#define LAST_REVISION_BLURB \
+_("The last revision number for the release.")
+
 #define MARCHING_ANTS_SPEED_BLURB \
 _("Speed of marching ants in the selection outline.  This value is in " \
   "milliseconds (less time indicates faster marching).")
@@ -223,6 +298,9 @@ _("Speed of marching ants in the selection outline.  This value is in " \
 _("GIMP will warn the user if an attempt is made to create an image that " \
   "would take more memory than the size specified here.")
 
+#define METADATA_ROTATION_POLICY_BLURB \
+_("How to handle \"Orientation\" metadata when opening a file.")
+
 #define MODULE_PATH_BLURB \
 "Sets the module search path."
 
@@ -239,6 +317,9 @@ _("Sets the monitor's vertical resolution, in dots per inch.  If set to " \
   "0, forces the X server to be queried for both horizontal and vertical " \
   "resolution information.")
 
+#define EDIT_NON_VISIBLE_BLURB \
+_("When enabled, non-visible layers can be edited as normal.")
+
 #define MOVE_TOOL_CHANGES_ACTIVE_BLURB \
 _("If enabled, the move tool sets the edited layer or path as active.  " \
   "This used to be the default behaviour in older versions.")
@@ -253,7 +334,7 @@ _("Sets the size of the navigation preview available in the lower right " \
   "corner of the image window.")
 
 #define NUM_PROCESSORS_BLURB \
-_("Sets how many processors GIMP should try to use simultaneously.")
+_("Sets how many threads GIMP should use for operations that support it.")
 
 #define PALETTE_PATH_BLURB \
 "Sets the palette search path."
@@ -279,6 +360,10 @@ _("Sets whether GIMP should create previews of layers and channels. " \
   "Previews in the layers and channels dialog are nice to have but they " \
   "can slow things down when working with large images.")
 
+#define GROUP_LAYER_PREVIEWS_BLURB \
+_("Sets whether GIMP should create previews of layer groups. " \
+  "Layer group previews are more expensive than ordinary layer previews.")
+
 #define LAYER_PREVIEW_SIZE_BLURB \
 _("Sets the preview size used for layers and channel previews in newly " \
   "created dialogs.")
@@ -330,14 +415,20 @@ _("Save the tool options when GIMP exits.")
 _("When enabled, all paint tools will show a preview of the current " \
   "brush's outline.")
 
+#define SNAP_BRUSH_OUTLINE_BLURB \
+_("When enabled, the brush outline will snap to individual dabs while " \
+  "painting.")
+
 #define SHOW_HELP_BUTTON_BLURB \
 _("When enabled, dialogs will show a help button that gives access to " \
   "the related help page.  Without this button, the help page can still " \
   "be reached by pressing F1.")
 
 #define SHOW_PAINT_TOOL_CURSOR_BLURB \
-_("When enabled, the mouse pointer will be shown over the image while " \
-  "using a paint tool.")
+_("When enabled, the pointer will be shown over the image while " \
+  "using a paint tool. "                                          \
+  "If both the brush outline and pointer are disabled, the "      \
+  "position will be indicated as unobtrusively as possibly.")
 
 #define SHOW_MENUBAR_BLURB \
 _("When enabled, the menubar is visible by default. This can also be " \
@@ -363,6 +454,10 @@ _("When enabled, the selection is visible by default. This can also be " \
 _("When enabled, the layer boundary is visible by default. This can also " \
   "be toggled with the \"View->Show Layer Boundary\" command.")
 
+#define SHOW_CANVAS_BOUNDARY_BLURB \
+_("When enabled, the canvas boundary is visible by default. This can also " \
+  "be toggled with the \"View->Show Canvas Boundary\" command.")
+
 #define SHOW_GUIDES_BLURB \
 _("When enabled, the guides are visible by default. This can also be " \
   "toggled with the \"View->Show Guides\" command.")
@@ -384,6 +479,9 @@ _("Use GIMP in a single-window mode.")
 #define HIDE_DOCKS_BLURB \
 _("Hide docks and other windows, leaving only image windows.")
 
+#define SHOW_TABS_BLURB \
+_("Show the image tabs bar in single window mode.")
+
 #define PLAYGROUND_NPD_TOOL_BLURB \
 _("Enable the N-Point Deformation tool.")
 
@@ -399,9 +497,15 @@ _("Enable the MyPaint Brush tool.")
 #define PLAYGROUND_SEAMLESS_CLONE_TOOL_BLURB \
 _("Enable the Seamless Clone tool.")
 
+#define PLAYGROUND_PAINT_SELECT_TOOL_BLURB \
+_("Enable the Paint Select tool.")
+
 #define SPACE_BAR_ACTION_BLURB \
 _("What to do when the space bar is pressed in the image window.")
 
+#define SWAP_COMPRESSION_BLURB \
+_("The compression method used for tile data stored in the swap file.")
+
 #define SWAP_PATH_BLURB \
 _("Sets the swap file location. GIMP uses a tile based memory allocation " \
   "scheme. The swap file is used to quickly and easily swap tiles out to " \
@@ -411,9 +515,6 @@ _("Sets the swap file location. GIMP uses a tile based memory allocation " \
   "a folder that is mounted over NFS.  For these reasons, it may be " \
   "desirable to put your swap file in \"/tmp\".")
 
-#define TEAROFF_MENUS_BLURB \
-_("When enabled, menus can be torn off.")
-
 #define CAN_CHANGE_ACCELS_BLURB \
 _("When enabled, you can change keyboard shortcuts for menu items " \
   "by hitting a key combination while the menu item is highlighted.")
@@ -431,7 +532,7 @@ _("Sets the folder for temporary storage. Files will appear here " \
   "is best if this folder not be one that is shared by other users.")
 
 #define THEME_BLURB \
-"The name of the theme to use."
+_("The name of the theme to use.")
 
 #define THEME_PATH_BLURB \
 "Sets the theme search path."
@@ -439,8 +540,8 @@ _("Sets the folder for temporary storage. Files will appear here " \
 #define ICON_THEME_BLURB \
 "The name of the icon theme to use."
 
-#define ICON_SIZE_BLURB \
-"The size of the icons to use."
+#define PREFER_SYMBOLIC_ICONS_BLURB \
+_("When enabled, symbolic icons will be preferred if available.")
 
 #define ICON_THEME_PATH_BLURB \
 "Sets the icon theme search path."
@@ -488,6 +589,9 @@ _("Sets the default set of layers to resize for the 'Canvas Size' dialog.")
 #define IMAGE_RESIZE_RESIZE_TEXT_LAYERS_BLURB \
 _("Sets the default 'Resize text layers' state for the 'Canvas Size' dialog.")
 
+# define ITEMS_SELECT_METHOD_BLURB \
+_("Sets how items are searched and selected from a textual pattern.")
+
 #define LAYER_NEW_NAME_BLURB \
 _("Sets the default layer name for the 'New Layer' dialog.")
 
@@ -554,6 +658,10 @@ _("Sets the default 'Scale imported paths to fit size' state for the 'Import Pat
 #define SELECTION_FEATHER_RADIUS_BLURB \
 _("Sets the default feather radius for the 'Feather Selection' dialog.")
 
+#define SELECTION_FEATHER_EDGE_LOCK_BLURB \
+_("Sets the default 'Selected areas continue outside the image' setting " \
+  "for the 'Feather Selection' dialog.")
+
 #define SELECTION_GROW_RADIUS_BLURB \
 _("Sets the default grow radius for the 'Grow Selection' dialog.")
 
@@ -599,11 +707,22 @@ _("Show the current foreground and background colors in the toolbox.")
 #define TOOLBOX_FOO_AREA_BLURB \
 _("Show the currently selected brush, pattern and gradient in the toolbox.")
 
+#define TOOLBOX_GROUPS_BLURB \
+_("Use a single toolbox button for grouped tools.")
+
 #define TOOLBOX_IMAGE_AREA_BLURB \
 _("Show the currently active image in the toolbox.")
 
 #define TOOLBOX_WILBER_BLURB \
-"Show the GIMP mascot at the top of the toolbox."
+_("Show the GIMP mascot at the top of the toolbox.")
+
+#define TRANSPARENCY_CUSTOM_COLOR1_BLURB \
+_("The first color to use in the transparency checkerboard, " \
+  "when Transparency Type is set to Custom colors.")
+
+#define TRANSPARENCY_CUSTOM_COLOR2_BLURB \
+_("The second color to use in the transparency checkerboard, " \
+  "when Transparency Type is set to Custom colors.")
 
 #define TRANSPARENCY_TYPE_BLURB \
 _("Sets the manner in which transparency is displayed in images.")
@@ -649,10 +768,10 @@ _("When enabled, uses OpenCL for some operations.")
 "offset problems turning it off helps."
 
 #define SEARCH_SHOW_UNAVAILABLE_BLURB \
-"When enabled, a search of actions will also return inactive actions."
+_("When enabled, a search of actions will also return inactive actions.")
 
 #define ACTION_HISTORY_SIZE_BLURB \
-"The maximum number of actions saved in history."
+_("The maximum number of actions saved in history.")
 
 
 #endif  /* __GIMP_RC_BLURBS_H__ */
