@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_PLUG_IN_H__
@@ -42,7 +42,7 @@ struct _GimpPlugIn
   GimpObject           parent_instance;
 
   GimpPlugInManager   *manager;
-  gchar               *prog;            /*  Plug-in's full path name          */
+  GFile               *file;            /*  Plug-in's full path name          */
 
   GimpPlugInCallMode   call_mode;       /*  QUERY, INIT or RUN                */
   guint                open : 1;        /*  Is the plug-in open?              */
@@ -82,7 +82,7 @@ GimpPlugIn  * gimp_plug_in_new               (GimpPlugInManager      *manager,
                                               GimpContext            *context,
                                               GimpProgress           *progress,
                                               GimpPlugInProcedure    *procedure,
-                                              const gchar            *prog);
+                                              GFile                  *file);
 
 gboolean      gimp_plug_in_open              (GimpPlugIn             *plug_in,
                                               GimpPlugInCallMode      call_mode,
@@ -105,10 +105,6 @@ void          gimp_plug_in_main_loop_quit    (GimpPlugIn             *plug_in);
 
 const gchar * gimp_plug_in_get_undo_desc     (GimpPlugIn             *plug_in);
 
-gboolean      gimp_plug_in_menu_register     (GimpPlugIn             *plug_in,
-                                              const gchar            *proc_name,
-                                              const gchar            *menu_path);
-
 void          gimp_plug_in_add_temp_proc     (GimpPlugIn             *plug_in,
                                               GimpTemporaryProcedure *procedure);
 void          gimp_plug_in_remove_temp_proc  (GimpPlugIn             *plug_in,
