@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -171,8 +171,8 @@ static gchar *
 sanity_check_glib (void)
 {
 #define GLIB_REQUIRED_MAJOR 2
-#define GLIB_REQUIRED_MINOR 36
-#define GLIB_REQUIRED_MICRO 0
+#define GLIB_REQUIRED_MINOR 56
+#define GLIB_REQUIRED_MICRO 2
 
   const gchar *mismatch = glib_check_version (GLIB_REQUIRED_MAJOR,
                                               GLIB_REQUIRED_MINOR,
@@ -204,8 +204,8 @@ static gchar *
 sanity_check_cairo (void)
 {
 #define CAIRO_REQUIRED_MAJOR 1
-#define CAIRO_REQUIRED_MINOR 12
-#define CAIRO_REQUIRED_MICRO 2
+#define CAIRO_REQUIRED_MINOR 14
+#define CAIRO_REQUIRED_MICRO 0
 
   if (cairo_version () < CAIRO_VERSION_ENCODE (CAIRO_REQUIRED_MAJOR,
                                                CAIRO_REQUIRED_MINOR,
@@ -234,8 +234,8 @@ static gchar *
 sanity_check_pango (void)
 {
 #define PANGO_REQUIRED_MAJOR 1
-#define PANGO_REQUIRED_MINOR 29
-#define PANGO_REQUIRED_MICRO 4
+#define PANGO_REQUIRED_MINOR 42
+#define PANGO_REQUIRED_MICRO 0
 
   const gchar *mismatch = pango_version_check (PANGO_REQUIRED_MAJOR,
                                                PANGO_REQUIRED_MINOR,
@@ -387,31 +387,15 @@ static gchar *
 sanity_check_lcms (void)
 {
 #define LCMS_REQUIRED_MAJOR 2
-#define LCMS_REQUIRED_MINOR 7
+#define LCMS_REQUIRED_MINOR 8
 
   gint lcms_version = cmsGetEncodedCMMversion ();
 
-  if (LCMS_VERSION > lcms_version)
-    {
-      return g_strdup_printf
-        ("Liblcms2 version mismatch!\n\n"
-         "GIMP was compiled against LittleCMS version %d.%d, but the\n"
-         "LittleCMS version found at runtime is only %d.%d.\n\n"
-         "Somehow you or your software packager managed\n"
-         "to install a LittleCMS that is older than what GIMP was\n"
-         "built against.\n\n"
-         "Please make sure that the installed LittleCMS version\n"
-         "is at least %d.%d and that headers and library match.",
-         LCMS_VERSION / 1000, LCMS_VERSION % 100 / 10,
-         lcms_version / 1000, lcms_version % 100 / 10,
-         LCMS_VERSION / 1000, LCMS_VERSION % 100 / 10);
-    }
-
   if (lcms_version < (LCMS_REQUIRED_MAJOR * 1000 +
                       LCMS_REQUIRED_MINOR * 10))
     {
       const gint lcms_major_version = lcms_version / 1000;
-      const gint lcms_minor_version = lcms_version % 100 / 10;
+      const gint lcms_minor_version = lcms_version % 1000 / 10;
 
       return g_strdup_printf
         ("Liblcms2 version too old!\n\n"
@@ -438,7 +422,7 @@ sanity_check_gexiv2 (void)
 
 #define GEXIV2_REQUIRED_MAJOR 0
 #define GEXIV2_REQUIRED_MINOR 10
-#define GEXIV2_REQUIRED_MICRO 3
+#define GEXIV2_REQUIRED_MICRO 6
 
   gint gexiv2_version = gexiv2_get_version ();
 
@@ -480,7 +464,7 @@ sanity_check_babl (void)
 
 #define BABL_REQUIRED_MAJOR 0
 #define BABL_REQUIRED_MINOR 1
-#define BABL_REQUIRED_MICRO 28
+#define BABL_REQUIRED_MICRO 78
 
   babl_get_version (&babl_major_version,
                     &babl_minor_version,
@@ -517,8 +501,8 @@ sanity_check_gegl (void)
   gint gegl_micro_version;
 
 #define GEGL_REQUIRED_MAJOR 0
-#define GEGL_REQUIRED_MINOR 3
-#define GEGL_REQUIRED_MICRO 18
+#define GEGL_REQUIRED_MINOR 4
+#define GEGL_REQUIRED_MICRO 38
 
   gegl_get_version (&gegl_major_version,
                     &gegl_minor_version,
@@ -605,6 +589,8 @@ sanity_check_gegl_ops (void)
   static const gchar *required_ops[] =
   {
     "gegl:alien-map",
+    "gegl:bayer-matrix",
+    "gegl:bloom",
     "gegl:buffer-sink",
     "gegl:buffer-source",
     "gegl:c2g",
@@ -638,6 +624,7 @@ sanity_check_gegl_ops (void)
     "gegl:engrave",
     "gegl:exposure",
     "gegl:fattal02",
+    "gegl:focus-blur",
     "gegl:fractal-trace",
     "gegl:gaussian-blur",
     "gegl:gaussian-blur-selective",
@@ -647,22 +634,27 @@ sanity_check_gegl_ops (void)
     "gegl:hue-chroma",
     "gegl:illusion",
     "gegl:image-gradient",
-    "gegl:introspect",
     "gegl:invert-gamma",
     "gegl:invert-linear",
+    "gegl:lens-blur",
     "gegl:lens-distortion",
     "gegl:lens-flare",
+    "gegl:linear-sinusoid",
+    "gegl:long-shadow",
     "gegl:mantiuk06",
     "gegl:map-absolute",
     "gegl:map-relative",
     "gegl:matting-global",
-/*  "gegl:matting-levin",*/ /* XXX: do we want to require this? */
     "gegl:maze",
+    "gegl:mean-curvature-blur",
+    "gegl:median-blur",
     "gegl:mirrors",
     "gegl:mono-mixer",
+    "gegl:mosaic",
     "gegl:motion-blur-circular",
     "gegl:motion-blur-linear",
     "gegl:motion-blur-zoom",
+    "gegl:newsprint",
     "gegl:noise-cie-lch",
     "gegl:noise-hsv",
     "gegl:noise-hurl",
@@ -671,6 +663,7 @@ sanity_check_gegl_ops (void)
     "gegl:noise-slur",
     "gegl:noise-solid",
     "gegl:noise-spread",
+    "gegl:normal-map",
     "gegl:npd",
     "gegl:oilify",
     "gegl:opacity",
@@ -680,6 +673,7 @@ sanity_check_gegl_ops (void)
     "gegl:photocopy",
     "gegl:pixelize",
     "gegl:polar-coordinates",
+    "gegl:recursive-transform",
     "gegl:red-eye-removal",
     "gegl:reinhard05",
     "gegl:rgb-clip",
@@ -688,6 +682,7 @@ sanity_check_gegl_ops (void)
     "gegl:scale-ratio",
     "gegl:seamless-clone",
     "gegl:sepia",
+    "gegl:shadows-highlights",
     "gegl:shift",
     "gegl:simplex-noise",
     "gegl:shift",
@@ -695,6 +690,9 @@ sanity_check_gegl_ops (void)
     "gegl:slic",
     "gegl:snn-mean",
     "gegl:softglow",
+    "gegl:spherize",
+    "gegl:spiral",
+    "gegl:stereographic-projection",
     "gegl:stretch-contrast",
     "gegl:stretch-contrast-hsv",
     "gegl:stress",
@@ -709,6 +707,7 @@ sanity_check_gegl_ops (void)
     "gegl:unsharp-mask",
     "gegl:value-invert",
     "gegl:value-propagate",
+    "gegl:variable-blur",
     "gegl:video-degradation",
     "gegl:vignette",
     "gegl:warp",
