@@ -12,14 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_GEGL_TOOL_H__
 #define __GIMP_GEGL_TOOL_H__
 
 
-#include "gimpimagemaptool.h"
+#include "gimpoperationtool.h"
 
 
 #define GIMP_TYPE_GEGL_TOOL            (gimp_gegl_tool_get_type ())
@@ -35,20 +35,16 @@ typedef struct _GimpGeglToolClass GimpGeglToolClass;
 
 struct _GimpGeglTool
 {
-  GimpImageMapTool  parent_instance;
-
-  gchar            *operation;
-  GimpObject       *config;
+  GimpOperationTool  parent_instance;
 
   /* dialog */
-  GtkWidget        *operation_combo;
-  GtkWidget        *options_frame;
-  GtkWidget        *options_table;
+  GtkWidget         *operation_combo;
+  GtkWidget         *description_label;
 };
 
 struct _GimpGeglToolClass
 {
-  GimpImageMapToolClass  parent_class;
+  GimpOperationToolClass  parent_class;
 };
 
 
