@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,23 +12,26 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <string.h>
+
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "widgets-types.h"
 
-#include "base/gimphistogram.h"
-
+#include "core/gimphistogram.h"
 #include "core/gimpmarshal.h"
 
 #include "gimphistogramview.h"
+#include "gimpwidgets-utils.h"
 
 
 #define MIN_WIDTH  64
@@ -50,38 +53,49 @@ enum
 };
 
 
-static void  gimp_histogram_view_finalize          (GObject        *object);
-static void  gimp_histogram_view_set_property      (GObject        *object,
-                                                    guint           property_id,
-                                                    const GValue   *value,
-                                                    GParamSpec     *pspec);
-static void  gimp_histogram_view_get_property      (GObject        *object,
-                                                    guint           property_id,
-                                                    GValue         *value,
-                                                    GParamSpec     *pspec);
+static void     gimp_histogram_view_dispose              (GObject        *object);
+static void     gimp_histogram_view_set_property         (GObject        *object,
+                                                          guint           property_id,
+                                                          const GValue   *value,
+                                                          GParamSpec     *pspec);
+static void     gimp_histogram_view_get_property         (GObject        *object,
+                                                          guint           property_id,
+                                                          GValue         *value,
+                                                          GParamSpec     *pspec);
 
-static void  gimp_histogram_view_size_request      (GtkWidget      *widget,
-                                                    GtkRequisition *requisition);
-static gboolean gimp_histogram_view_expose         (GtkWidget      *widget,
-                                                    GdkEventExpose *event);
-static gboolean gimp_histogram_view_button_press   (GtkWidget      *widget,
-                                                    GdkEventButton *bevent);
-static gboolean gimp_histogram_view_button_release (GtkWidget      *widget,
-                                                    GdkEventButton *bevent);
-static gboolean gimp_histogram_view_motion_notify  (GtkWidget      *widget,
-                                                    GdkEventMotion *bevent);
+static void     gimp_histogram_view_get_preferred_width  (GtkWidget      *widget,
+                                                          gint           *minimum_width,
+                                                          gint           *natural_width);
+static void     gimp_histogram_view_get_preferred_height (GtkWidget      *widget,
+                                                          gint           *minimum_height,
+                                                          gint           *natural_height);
+static gboolean gimp_histogram_view_draw                 (GtkWidget      *widget,
+                                                          cairo_t        *cr);
+static gboolean gimp_histogram_view_button_press         (GtkWidget      *widget,
+                                                          GdkEventButton *bevent);
+static gboolean gimp_histogram_view_button_release       (GtkWidget      *widget,
+                                                          GdkEventButton *bevent);
+static gboolean gimp_histogram_view_motion_notify        (GtkWidget      *widget,
+                                                          GdkEventMotion *bevent);
 
-static void     gimp_histogram_view_draw_spike (GimpHistogramView    *view,
-                                                GimpHistogramChannel  channel,
-                                                GdkGC                *gc,
-                                                GdkGC                *bg_gc,
-                                                gint                  x,
-                                                gint                  i,
-                                                gint                  j,
-                                                gdouble               max,
-                                                gdouble               bg_max,
-                                                gint                  height,
-                                                gint                  border);
+static void     gimp_histogram_view_notify      (GimpHistogram        *histogram,
+                                                 const GParamSpec     *pspec,
+                                                 GimpHistogramView    *view);
+static void     gimp_histogram_view_update_bins (GimpHistogramView    *view);
+
+static void     gimp_histogram_view_draw_spike  (GimpHistogramView    *view,
+                                                 GimpHistogramChannel  channel,
+                                                 cairo_t              *cr,
+                                                 const GdkRGBA        *fg_color,
+                                                 cairo_operator_t      fg_operator,
+                                                 const GdkRGBA        *bg_color,
+                                                 gint                  x,
+                                                 gint                  i,
+                                                 gint                  j,
+                                                 gdouble               max,
+                                                 gdouble               bg_max,
+                                                 gint                  height,
+                                                 gint                  border);
 
 
 G_DEFINE_TYPE (GimpHistogramView, gimp_histogram_view,
@@ -109,12 +123,13 @@ gimp_histogram_view_class_init (GimpHistogramViewClass *klass)
                   G_TYPE_INT,
                   G_TYPE_INT);
 
-  object_class->finalize             = gimp_histogram_view_finalize;
+  object_class->dispose              = gimp_histogram_view_dispose;
   object_class->get_property         = gimp_histogram_view_get_property;
   object_class->set_property         = gimp_histogram_view_set_property;
 
-  widget_class->size_request         = gimp_histogram_view_size_request;
-  widget_class->expose_event         = gimp_histogram_view_expose;
+  widget_class->get_preferred_width  = gimp_histogram_view_get_preferred_width;
+  widget_class->get_preferred_height = gimp_histogram_view_get_preferred_height;
+  widget_class->draw                 = gimp_histogram_view_draw;
   widget_class->button_press_event   = gimp_histogram_view_button_press;
   widget_class->button_release_event = gimp_histogram_view_button_release;
   widget_class->motion_notify_event  = gimp_histogram_view_motion_notify;
@@ -156,28 +171,20 @@ gimp_histogram_view_init (GimpHistogramView *view)
 {
   view->histogram    = NULL;
   view->bg_histogram = NULL;
+  view->n_bins       = 256;
   view->start        = 0;
   view->end          = 255;
 }
 
 static void
-gimp_histogram_view_finalize (GObject *object)
+gimp_histogram_view_dispose (GObject *object)
 {
   GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (object);
 
-  if (view->histogram)
-    {
-      gimp_histogram_unref (view->histogram);
-      view->histogram = NULL;
-    }
+  gimp_histogram_view_set_histogram (view, NULL);
+  gimp_histogram_view_set_background (view, NULL);
 
-  if (view->bg_histogram)
-    {
-      gimp_histogram_unref (view->bg_histogram);
-      view->bg_histogram = NULL;
-    }
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -243,13 +250,23 @@ gimp_histogram_view_get_property (GObject      *object,
 }
 
 static void
-gimp_histogram_view_size_request (GtkWidget      *widget,
-                                  GtkRequisition *requisition)
+gimp_histogram_view_get_preferred_width (GtkWidget *widget,
+                                         gint      *minimum_width,
+                                         gint      *natural_width)
 {
   GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (widget);
 
-  requisition->width  = MIN_WIDTH  + 2 * view->border_width;
-  requisition->height = MIN_HEIGHT + 2 * view->border_width;
+  *minimum_width = *natural_width = MIN_WIDTH + 2 * view->border_width;
+}
+
+static void
+gimp_histogram_view_get_preferred_height (GtkWidget *widget,
+                                          gint      *minimum_height,
+                                          gint      *natural_height)
+{
+  GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (widget);
+
+  *minimum_height = *natural_height = MIN_HEIGHT + 2 * view->border_width;
 }
 
 static gdouble
@@ -276,11 +293,12 @@ gimp_histogram_view_get_maximum (GimpHistogramView    *view,
 }
 
 static gboolean
-gimp_histogram_view_expose (GtkWidget      *widget,
-                            GdkEventExpose *event)
+gimp_histogram_view_draw (GtkWidget *widget,
+                          cairo_t   *cr)
 {
   GimpHistogramView *view  = GIMP_HISTOGRAM_VIEW (widget);
-  GtkStyle          *style = gtk_widget_get_style (widget);
+  GtkStyleContext   *style = gtk_widget_get_style_context (widget);
+  GtkAllocation      allocation;
   gint               x;
   gint               x1, x2;
   gint               border;
@@ -288,33 +306,47 @@ gimp_histogram_view_expose (GtkWidget      *widget,
   gdouble            max    = 0.0;
   gdouble            bg_max = 0.0;
   gint               xstop;
-  GdkGC             *gc_in;
-  GdkGC             *gc_out;
-  GdkGC             *bg_gc_in;
-  GdkGC             *bg_gc_out;
-  GdkGC             *rgb_gc[3]  = { NULL, NULL, NULL };
+  GdkRGBA            grid_color;
+  GdkRGBA            color_in;
+  GdkRGBA            color_out;
+  GdkRGBA            bg_color_in;
+  GdkRGBA            bg_color_out;
+  GdkRGBA            rgb_color[3];
 
-  if (! view->histogram && ! view->bg_histogram)
-    return FALSE;
+  gtk_widget_get_allocation (widget, &allocation);
+
+  gtk_style_context_save (style);
+  gtk_style_context_add_class (style, "view");
+
+  gtk_render_background (style, cr, 0, 0,
+                         allocation.width, allocation.height);
 
   border = view->border_width;
-  width  = widget->allocation.width  - 2 * border;
-  height = widget->allocation.height - 2 * border;
+  width  = allocation.width  - 2 * border;
+  height = allocation.height - 2 * border;
 
-  x1 = CLAMP (MIN (view->start, view->end), 0, 255);
-  x2 = CLAMP (MAX (view->start, view->end), 0, 255);
-
-  gdk_draw_rectangle (widget->window,
-                      style->base_gc[GTK_STATE_NORMAL], TRUE,
-                      0, 0,
-                      widget->allocation.width,
-                      widget->allocation.height);
+  cairo_set_line_width (cr, 1.0);
+  cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
+  cairo_translate (cr, 0.5, 0.5);
 
   /*  Draw the outer border  */
-  gdk_draw_rectangle (widget->window,
-                      style->dark_gc[GTK_STATE_NORMAL], FALSE,
-                      border, border,
-                      width - 1, height - 1);
+  gtk_style_context_add_class (style, "grid");
+  gtk_style_context_get_color (style, gtk_style_context_get_state (style),
+                               &grid_color);
+  gtk_style_context_remove_class (style, "grid");
+
+  gdk_cairo_set_source_rgba (cr, &grid_color);
+  cairo_rectangle (cr, border, border, width - 1, height - 1);
+  cairo_stroke (cr);
+
+  if (! view->histogram && ! view->bg_histogram)
+    {
+      gtk_style_context_restore (style);
+      return FALSE;
+    }
+
+  x1 = CLAMP (MIN (view->start, view->end), 0, view->n_bins - 1);
+  x2 = CLAMP (MAX (view->start, view->end), 0, view->n_bins - 1);
 
   if (view->histogram)
     max = gimp_histogram_view_get_maximum (view, view->histogram,
@@ -324,29 +356,27 @@ gimp_histogram_view_expose (GtkWidget      *widget,
     bg_max = gimp_histogram_view_get_maximum (view, view->bg_histogram,
                                               view->channel);
 
-  gc_in  = style->text_gc[GTK_STATE_SELECTED];
-  gc_out = style->text_gc[GTK_STATE_NORMAL];
+  gtk_style_context_get_color (style, gtk_style_context_get_state (style),
+                               &color_out);
+  bg_color_out       = color_out;
+  bg_color_out.alpha = 0.5;
 
-  bg_gc_in  = style->mid_gc[GTK_STATE_SELECTED];
-  bg_gc_out = style->mid_gc[GTK_STATE_NORMAL];
+  gtk_style_context_save (style);
+  gtk_style_context_set_state (style, GTK_STATE_FLAG_SELECTED);
+  gtk_style_context_get_color (style, gtk_style_context_get_state (style),
+                               &color_in);
+  bg_color_in       = color_in;
+  bg_color_in.alpha = 0.5;
+  gtk_style_context_restore (style);
 
   if (view->channel == GIMP_HISTOGRAM_RGB)
     {
-      GdkGCValues  values;
-      GdkColor     color;
-
-      values.function = GDK_OR;
-
       for (x = 0; x < 3; x++)
         {
-          rgb_gc[x] = gdk_gc_new_with_values (widget->window,
-                                              &values, GDK_GC_FUNCTION);
-
-          color.red   = (x == 0 ? 0xFFFF : 0x0);
-          color.green = (x == 1 ? 0xFFFF : 0x0);
-          color.blue  = (x == 2 ? 0xFFFF : 0x0);
-
-          gdk_gc_set_rgb_fg_color (rgb_gc[x], &color);
+          rgb_color[x].red   = (x == 0 ? 1.0 : 0.0);
+          rgb_color[x].green = (x == 1 ? 1.0 : 0.0);
+          rgb_color[x].blue  = (x == 2 ? 1.0 : 0.0);
+          rgb_color[x].alpha = 1.0;
         }
     }
 
@@ -355,10 +385,10 @@ gimp_histogram_view_expose (GtkWidget      *widget,
     {
       gboolean  in_selection = FALSE;
 
-      gint  i = (x * 256) / width;
-      gint  j = ((x + 1) * 256) / width;
+      gint  i = (x * view->n_bins) / width;
+      gint  j = ((x + 1) * view->n_bins) / width;
 
-      if (! (x1 == 0 && x2 == 255))
+      if (! (x1 == 0 && x2 == (view->n_bins - 1)))
         {
           gint k = i;
 
@@ -369,55 +399,62 @@ gimp_histogram_view_expose (GtkWidget      *widget,
 
       if (view->subdivisions > 1 && x >= (xstop * width / view->subdivisions))
         {
-          gdk_draw_line (widget->window,
-                         style->dark_gc[GTK_STATE_NORMAL],
-                         x + border, border,
-                         x + border, border + height - 1);
+          gdk_cairo_set_source_rgba (cr, &grid_color);
+
+          cairo_move_to (cr, x + border, border);
+          cairo_line_to (cr, x + border, border + height - 1);
+          cairo_stroke (cr);
+
           xstop++;
         }
       else if (in_selection)
         {
-          gdk_draw_line (widget->window,
-                         style->base_gc[GTK_STATE_SELECTED],
-                         x + border, border,
-                         x + border, border + height - 1);
+          gtk_style_context_save (style);
+          gtk_style_context_set_state (style, GTK_STATE_FLAG_SELECTED);
+
+          gtk_render_background (style, cr,
+                                 x + border, border,
+                                 1,          height - 1);
+
+          gtk_style_context_restore (style);
         }
 
       if (view->channel == GIMP_HISTOGRAM_RGB)
         {
-          gint c;
+          GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
+          gint    c;
 
           for (c = 0; c < 3; c++)
-            gimp_histogram_view_draw_spike (view, GIMP_HISTOGRAM_RED + c,
-                                            style->black_gc,
+            gimp_histogram_view_draw_spike (view, GIMP_HISTOGRAM_RED + c, cr,
+                                            &black,
+                                            CAIRO_OPERATOR_OVER,
                                             NULL,
                                             x, i, j, max, bg_max, height, border);
 
           for (c = 0; c < 3; c++)
-            gimp_histogram_view_draw_spike (view, GIMP_HISTOGRAM_RED + c,
-                                            rgb_gc[c],
+            gimp_histogram_view_draw_spike (view, GIMP_HISTOGRAM_RED + c, cr,
+                                            &rgb_color[c],
+                                            CAIRO_OPERATOR_ADD,
                                             NULL,
                                             x, i, j, max, bg_max, height, border);
 
-          gimp_histogram_view_draw_spike (view, view->channel,
-                                          in_selection ? gc_in : gc_out,
+          gimp_histogram_view_draw_spike (view, view->channel, cr,
+                                          in_selection ? &color_in : &color_out,
+                                          CAIRO_OPERATOR_OVER,
                                           NULL,
                                           x, i, j, max, bg_max, height, border);
         }
       else
         {
-          gimp_histogram_view_draw_spike (view, view->channel,
-                                          in_selection ? gc_in : gc_out,
-                                          in_selection ? bg_gc_in  : bg_gc_out,
+          gimp_histogram_view_draw_spike (view, view->channel, cr,
+                                          in_selection ? &color_in : &color_out,
+                                          CAIRO_OPERATOR_OVER,
+                                          in_selection ? &bg_color_in : &bg_color_out,
                                           x, i, j, max, bg_max, height, border);
         }
     }
 
-  if (view->channel == GIMP_HISTOGRAM_RGB)
-    {
-      for (x = 0; x < 3; x++)
-        g_object_unref (rgb_gc[x]);
-    }
+  gtk_style_context_restore (style);
 
   return FALSE;
 }
@@ -425,8 +462,10 @@ gimp_histogram_view_expose (GtkWidget      *widget,
 static void
 gimp_histogram_view_draw_spike (GimpHistogramView    *view,
                                 GimpHistogramChannel  channel,
-                                GdkGC                *gc,
-                                GdkGC                *bg_gc,
+                                cairo_t              *cr,
+                                const GdkRGBA        *fg_color,
+                                cairo_operator_t      fg_operator,
+                                const GdkRGBA        *bg_color,
                                 gint                  x,
                                 gint                  i,
                                 gint                  j,
@@ -435,33 +474,39 @@ gimp_histogram_view_draw_spike (GimpHistogramView    *view,
                                 gint                  height,
                                 gint                  border)
 {
-  gdouble  value    = 0.0;
-  gdouble  bg_value = 0.0;
-  gint     y;
-  gint     bg_y;
+  gdouble value    = 0.0;
+  gdouble bg_value = 0.0;
+  gint    y;
+  gint    bg_y;
 
   if (view->histogram)
     {
+      gint ii = i;
+
       do
         {
-          gdouble v = gimp_histogram_get_value (view->histogram, channel, i++);
+          gdouble v = gimp_histogram_get_value (view->histogram,
+                                                channel, ii++);
 
           if (v > value)
             value = v;
         }
-      while (i < j);
+      while (ii < j);
     }
 
-  if (bg_gc && view->bg_histogram)
+  if (bg_color && view->bg_histogram)
     {
+      gint ii = i;
+
       do
         {
-          gdouble v = gimp_histogram_get_value (view->bg_histogram, channel, i++);
+          gdouble v = gimp_histogram_get_value (view->bg_histogram,
+                                                channel, ii++);
 
           if (v > bg_value)
             bg_value = v;
         }
-      while (i < j);
+      while (ii < j);
     }
 
   if (value <= 0.0 && bg_value <= 0.0)
@@ -485,14 +530,26 @@ gimp_histogram_view_draw_spike (GimpHistogramView    *view,
       break;
     }
 
-  if (bg_gc)
-    gdk_draw_line (GTK_WIDGET (view)->window, bg_gc,
-                   x + border, height + border - 1,
-                   x + border, height + border - bg_y - 1);
+  if (bg_color)
+    {
+      gdk_cairo_set_source_rgba (cr, bg_color);
 
-  gdk_draw_line (GTK_WIDGET (view)->window, gc,
-                 x + border, height + border - 1,
-                 x + border, height + border - y - 1);
+      cairo_move_to (cr, x + border, height + border - 1);
+      cairo_line_to (cr, x + border, height + border - bg_y - 1);
+
+      cairo_stroke (cr);
+    }
+
+  cairo_set_operator (cr, fg_operator);
+
+  gdk_cairo_set_source_rgba (cr, fg_color);
+
+  cairo_move_to (cr, x + border, height + border - 1);
+  cairo_line_to (cr, x + border, height + border - y - 1);
+
+  cairo_stroke (cr);
+
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 }
 
 static gboolean
@@ -501,18 +558,29 @@ gimp_histogram_view_button_press (GtkWidget      *widget,
 {
   GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (widget);
 
-  if (bevent->type == GDK_BUTTON_PRESS && bevent->button == 1)
+  if (! view->grab_seat &&
+      bevent->type == GDK_BUTTON_PRESS && bevent->button == 1)
     {
-      gint width;
+      GdkSeat       *seat = gdk_event_get_seat ((GdkEvent *) bevent);
+      GtkAllocation  allocation;
+      gint           width;
 
-      gdk_pointer_grab (widget->window, FALSE,
-                        GDK_BUTTON_RELEASE_MASK | GDK_BUTTON1_MOTION_MASK,
-                        NULL, NULL, bevent->time);
+      if (gdk_seat_grab (seat, gtk_widget_get_window (widget),
+                         GDK_SEAT_CAPABILITY_ALL, FALSE,
+                         NULL, (GdkEvent *) bevent,
+                         NULL, NULL) != GDK_GRAB_SUCCESS)
+        {
+          return TRUE;
+        }
 
-      width = widget->allocation.width - 2 * view->border_width;
+      view->grab_seat = seat;
 
-      view->start = CLAMP ((((bevent->x - view->border_width) * 256) / width),
-                           0, 255);
+      gtk_widget_get_allocation (widget, &allocation);
+
+      width = allocation.width - 2 * view->border_width;
+
+      view->start = CLAMP (((bevent->x - view->border_width) * view->n_bins) / width,
+                           0, view->n_bins - 1);
       view->end   = view->start;
 
       gtk_widget_queue_draw (widget);
@@ -527,12 +595,13 @@ gimp_histogram_view_button_release (GtkWidget      *widget,
 {
   GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (widget);
 
-  if (bevent->button == 1)
+  if (gdk_event_get_seat ((GdkEvent *) bevent) == view->grab_seat &&
+      bevent->button == 1)
     {
       gint start, end;
 
-      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (view)),
-                                  bevent->time);
+      gdk_seat_ungrab (view->grab_seat);
+      view->grab_seat = NULL;
 
       start = view->start;
       end   = view->end;
@@ -552,18 +621,27 @@ gimp_histogram_view_motion_notify (GtkWidget      *widget,
                                    GdkEventMotion *mevent)
 {
   GimpHistogramView *view = GIMP_HISTOGRAM_VIEW (widget);
+  GtkAllocation      allocation;
   gint               width;
 
-  width = widget->allocation.width - 2 * view->border_width;
+  if (gdk_event_get_seat ((GdkEvent *) mevent) == view->grab_seat)
+    {
+      gtk_widget_get_allocation (widget, &allocation);
 
-  view->start = CLAMP ((((mevent->x - view->border_width) * 256) / width),
-                       0, 255);
+      width = allocation.width - 2 * view->border_width;
 
-  gtk_widget_queue_draw (widget);
+      view->start = CLAMP (((mevent->x - view->border_width) * view->n_bins) / width,
+                           0, view->n_bins - 1);
+
+      gtk_widget_queue_draw (widget);
+    }
 
   return TRUE;
 }
 
+
+/*  public functions  */
+
 GtkWidget *
 gimp_histogram_view_new (gboolean range)
 {
@@ -586,24 +664,35 @@ gimp_histogram_view_set_histogram (GimpHistogramView *view,
 #if 0
   g_return_if_fail (histogram == NULL ||
                     view->bg_histogram == NULL ||
-                    gimp_histogram_n_channels (view->bg_histogram) ==
-                    gimp_histogram_n_channels (histogram));
+                    gimp_histogram_n_components (view->bg_histogram) ==
+                    gimp_histogram_n_components (histogram));
 #endif
 
   if (view->histogram != histogram)
     {
       if (view->histogram)
-        gimp_histogram_unref (view->histogram);
+        {
+          g_signal_handlers_disconnect_by_func (view->histogram,
+                                                gimp_histogram_view_notify,
+                                                view);
+          g_object_unref (view->histogram);
+        }
 
       view->histogram = histogram;
 
       if (histogram)
         {
-          gimp_histogram_ref (histogram);
+          g_object_ref (histogram);
 
-          if (view->channel >= gimp_histogram_n_channels (histogram))
+          g_signal_connect (histogram, "notify",
+                            G_CALLBACK (gimp_histogram_view_notify),
+                            view);
+
+          if (! gimp_histogram_has_channel (histogram, view->channel))
             gimp_histogram_view_set_channel (view, GIMP_HISTOGRAM_VALUE);
         }
+
+      gimp_histogram_view_update_bins (view);
     }
 
   gtk_widget_queue_draw (GTK_WIDGET (view));
@@ -625,24 +714,35 @@ gimp_histogram_view_set_background (GimpHistogramView *view,
 #if 0
   g_return_if_fail (histogram == NULL ||
                     view->histogram == NULL ||
-                    gimp_histogram_n_channels (view->histogram) ==
-                    gimp_histogram_n_channels (histogram));
+                    gimp_histogram_n_components (view->histogram) ==
+                    gimp_histogram_n_components (histogram));
 #endif
 
   if (view->bg_histogram != histogram)
     {
       if (view->bg_histogram)
-        gimp_histogram_unref (view->bg_histogram);
+        {
+          g_signal_handlers_disconnect_by_func (view->bg_histogram,
+                                                gimp_histogram_view_notify,
+                                                view);
+          g_object_unref (view->bg_histogram);
+        }
 
       view->bg_histogram = histogram;
 
       if (histogram)
         {
-          gimp_histogram_ref (histogram);
+          g_object_ref (histogram);
 
-          if (view->channel >= gimp_histogram_n_channels (histogram))
+          g_signal_connect (histogram, "notify",
+                            G_CALLBACK (gimp_histogram_view_notify),
+                            view);
+
+          if (! gimp_histogram_has_channel (histogram, view->channel))
             gimp_histogram_view_set_channel (view, GIMP_HISTOGRAM_VALUE);
         }
+
+      gimp_histogram_view_update_bins (view);
     }
 
   gtk_widget_queue_draw (GTK_WIDGET (view));
@@ -722,3 +822,47 @@ gimp_histogram_view_get_range (GimpHistogramView *view,
   if (start) *start = view->start;
   if (end)   *end   = view->end;
 }
+
+
+/*  private functions  */
+
+static void
+gimp_histogram_view_notify (GimpHistogram     *histogram,
+                            const GParamSpec  *pspec,
+                            GimpHistogramView *view)
+{
+  if (! strcmp (pspec->name, "n-bins"))
+    {
+      gimp_histogram_view_update_bins (view);
+    }
+  else
+    {
+      gtk_widget_queue_draw (GTK_WIDGET (view));
+    }
+}
+
+static void
+gimp_histogram_view_update_bins (GimpHistogramView *view)
+{
+  gint new_bins = 0;
+
+  if (view->histogram)
+    new_bins = gimp_histogram_n_bins (view->histogram);
+  else if (view->bg_histogram)
+    new_bins = gimp_histogram_n_bins (view->bg_histogram);
+
+  if (new_bins > 0 && new_bins != view->n_bins)
+    {
+      view->start = MIN (ROUND ((gdouble) view->start *
+                                new_bins / view->n_bins),
+                         new_bins - 1);
+      view->end   = MAX (ROUND ((gdouble) (view->end + 1) *
+                                new_bins / view->n_bins) - 1,
+                         0);
+
+      view->n_bins = new_bins;
+
+      g_signal_emit (view, histogram_view_signals[RANGE_CHANGED], 0,
+                     view->start, view->end);
+    }
+}
