@@ -7,10 +7,10 @@
  * Some of the code in here was inspired and partly copied from pango
  * and librsvg.
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,9 +18,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,6 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <babl/babl.h>
 #include <glib-object.h>
 
 #include "gimpcolortypes.h"
@@ -213,18 +213,19 @@ static const ColorEntry named_colors[] =
 /**
  * gimp_rgb_parse_name:
  * @rgb:  a #GimpRGB struct used to return the parsed color
- * @name: a color name (in UTF-8 encoding)
+ * @name: (array length=len): a color name (in UTF-8 encoding)
  * @len:  the length of @name, in bytes. or -1 if @name is nul-terminated
  *
  * Attempts to parse a color name. This function accepts <ulink
- * url="http://www.w3.org/TR/SVG/types.html">SVG 1.0 color keywords</ulink>.
+ * url="https://www.w3.org/TR/SVG/types.html#ColorKeywords">SVG 1.0
+ * color keywords</ulink>.
  *
  * This function does not touch the alpha component of @rgb.
  *
- * Return value: %TRUE if @name was parsed successfully and @rgb has
+ * Returns: %TRUE if @name was parsed successfully and @rgb has
  *               been set, %FALSE otherwise
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_rgb_parse_name (GimpRGB     *rgb,
@@ -249,7 +250,7 @@ gimp_rgb_parse_name (GimpRGB     *rgb,
 /**
  * gimp_rgb_parse_hex:
  * @rgb: a #GimpRGB struct used to return the parsed color
- * @hex: a string describing a color in hexadecimal notation
+ * @hex: (array length=len): a string describing a color in hexadecimal notation
  * @len: the length of @hex, in bytes. or -1 if @hex is nul-terminated
  *
  * Attempts to parse a string describing an RGB color in hexadecimal
@@ -257,10 +258,10 @@ gimp_rgb_parse_name (GimpRGB     *rgb,
  *
  * This function does not touch the alpha component of @rgb.
  *
- * Return value: %TRUE if @hex was parsed successfully and @rgb has
+ * Returns: %TRUE if @hex was parsed successfully and @rgb has
  *               been set, %FALSE otherwise
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_rgb_parse_hex (GimpRGB     *rgb,
@@ -285,8 +286,8 @@ gimp_rgb_parse_hex (GimpRGB     *rgb,
 /**
  * gimp_rgb_parse_css:
  * @rgb: a #GimpRGB struct used to return the parsed color
- * @css: a string describing a color in CSS notation
- * @len: the length of @hex, in bytes. or -1 if @hex is nul-terminated
+ * @css: (array length=len): a string describing a color in CSS notation
+ * @len: the length of @css, in bytes. or -1 if @css is nul-terminated
  *
  * Attempts to parse a string describing an RGB color in CSS
  * notation. This can be either a numerical representation
@@ -297,10 +298,10 @@ gimp_rgb_parse_hex (GimpRGB     *rgb,
  *
  * This function does not touch the alpha component of @rgb.
  *
- * Return value: %TRUE if @css was parsed successfully and @rgb has been
+ * Returns: %TRUE if @css was parsed successfully and @rgb has been
  *               set, %FALSE otherwise
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_rgb_parse_css (GimpRGB     *rgb,
@@ -325,8 +326,8 @@ gimp_rgb_parse_css (GimpRGB     *rgb,
 /**
  * gimp_rgba_parse_css:
  * @rgba: a #GimpRGB struct used to return the parsed color
- * @css: a string describing a color in CSS notation
- * @len: the length of @hex, in bytes. or -1 if @hex is nul-terminated
+ * @css: (array length=len): a string describing a color in CSS notation
+ * @len: the length of @css, in bytes. or -1 if @css is nul-terminated
  *
  * Similar to gimp_rgb_parse_css() but handles RGB colors with alpha
  * channel in the numerical CSS notation (<code>rgba(255,0,0,255)</code>
@@ -335,10 +336,10 @@ gimp_rgb_parse_css (GimpRGB     *rgb,
  * It doesn't handle the hexadecimal notation or color names because
  * they leave the alpha channel unspecified.
  *
- * Return value: %TRUE if @css was parsed successfully and @rgb has been
+ * Returns: %TRUE if @css was parsed successfully and @rgb has been
  *               set, %FALSE otherwise
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_rgba_parse_css (GimpRGB     *rgba,
@@ -374,33 +375,34 @@ gimp_rgba_parse_css (GimpRGB     *rgba,
 
 /**
  * gimp_rgb_list_names:
- * @names:  return location for an array of color names
- * @colors: return location for an array of GimpRGB structs
+ * @names: (out) (array length=n_colors) (transfer container): return location for an array of color names
+ * @colors: (out) (array length=n_colors) (transfer container): return location for an array of GimpRGB structs
+ * @n_colors: (out): The number of named colors
  *
- * Allows to retrieve the list of <ulink
- * url="http://www.w3.org/TR/SVG/types.html">SVG 1.0 color
+ * Returns the list of <ulink
+ * url="https://www.w3.org/TR/SVG/types.html">SVG 1.0 color
  * keywords</ulink> that is used by gimp_rgb_parse_name().
  *
  * The returned strings are const and must not be freed. Only the two
  * arrays are allocated dynamically. You must call g_free() on the
  * @names and @colors arrays when they are not any longer needed.
  *
- * Return value: the number of named colors
- *               (i.e. the length of the returned arrays)
- *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
-gint
+void
 gimp_rgb_list_names (const gchar ***names,
-                     GimpRGB      **colors)
+                     GimpRGB      **colors,
+                     gint          *n_colors)
 {
   gint i;
 
-  g_return_val_if_fail (names != NULL, 0);
-  g_return_val_if_fail (colors != NULL, 0);
+  g_return_if_fail (names != NULL);
+  g_return_if_fail (colors != NULL);
+  g_return_if_fail (n_colors != NULL);
 
   *names  = g_new (const gchar *, G_N_ELEMENTS (named_colors));
   *colors = g_new (GimpRGB, G_N_ELEMENTS (named_colors));
+  *n_colors = G_N_ELEMENTS (named_colors);
 
   for (i = 0; i < G_N_ELEMENTS (named_colors); i++)
     {
@@ -412,11 +414,8 @@ gimp_rgb_list_names (const gchar ***names,
                            named_colors[i].blue,
                            0xFF);
     }
-
-  return G_N_ELEMENTS (named_colors);
 }
 
-
 static gchar *
 gimp_rgb_parse_strip (const gchar *str,
                       gint         len)
