@@ -8,9 +8,9 @@
  *
  * Copyright (C) 1997 Andy Thomas  alt@picnic.demon.co.uk
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,10 +19,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * A fair proprotion of this code was taken from the Whirl plug-in
+ * A fair proportion of this code was taken from the Whirl plug-in
  * which was copyrighted by Federico Mena Quintero (as below).
  *
  * Whirl plug-in --- distort an image into a whirlpool
@@ -43,135 +42,207 @@
 
 #define PLUG_IN_PROC   "plug-in-blinds"
 #define PLUG_IN_BINARY "blinds"
-
-#define SCALE_WIDTH    150
+#define PLUG_IN_ROLE   "gimp-blinds"
 
 #define MAX_FANS       100
 
-#define HORIZONTAL       0
-#define VERTICAL         1
-
 /* Variables set in dialog box */
 typedef struct data
 {
-  gint     angledsp;
-  gint     numsegs;
-  gint     orientation;
+  gint                 angledsp;
+  gint                 numsegs;
+  GimpOrientationType  orientation;
   gboolean bg_trans;
 } BlindVals;
 
+
+typedef struct _Blinds      Blinds;
+typedef struct _BlindsClass BlindsClass;
+
+struct _Blinds
+{
+  GimpPlugIn parent_instance;
+};
+
+struct _BlindsClass
+{
+  GimpPlugInClass parent_class;
+};
+
+
+#define BLINDS_TYPE  (blinds_get_type ())
+#define BLINDS (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BLINDS_TYPE, Blinds))
+
+GType                   blinds_get_type         (void) G_GNUC_CONST;
+
+static GList          * blinds_query_procedures (GimpPlugIn           *plug_in);
+static GimpProcedure  * blinds_create_procedure (GimpPlugIn           *plug_in,
+                                                 const gchar          *name);
+
+static GimpValueArray * blinds_run              (GimpProcedure        *procedure,
+                                                 GimpRunMode           run_mode,
+                                                 GimpImage            *image,
+                                                 gint                  n_drawables,
+                                                 GimpDrawable        **drawables,
+                                                 const GimpValueArray *args,
+                                                 gpointer              run_data);
+
+static gboolean         blinds_dialog           (GimpDrawable         *drawable);
+static void       blinds_scale_entry_update_int (GimpLabelSpin        *entry,
+                                                 gint                 *value);
+
+static void             dialog_update_preview   (GimpDrawable         *drawable,
+                                                 GimpPreview          *preview);
+static void             apply_blinds            (GimpDrawable         *drawable);
+
+
+G_DEFINE_TYPE (Blinds, blinds, GIMP_TYPE_PLUG_IN)
+
+GIMP_MAIN (BLINDS_TYPE)
+DEFINE_STD_SET_I18N
+
+
 /* Array to hold each size of fans. And no there are not each the
  * same size (rounding errors...)
  */
 
 static gint fanwidths[MAX_FANS];
 
-static void      query  (void);
-static void      run    (const gchar      *name,
-                         gint              nparams,
-                         const GimpParam  *param,
-                         gint             *nreturn_vals,
-                         GimpParam       **return_vals);
-
-static gboolean  blinds_dialog         (GimpDrawable  *drawable);
-
-static void      dialog_update_preview (GimpDrawable  *drawable,
-                                        GimpPreview   *preview);
-static void      apply_blinds          (GimpDrawable  *drawable);
-
-const GimpPlugInInfo PLUG_IN_INFO =
-{
-  NULL,    /* init_proc */
-  NULL,    /* quit_proc */
-  query,   /* query_proc */
-  run,     /* run_proc */
-};
-
 /* Values when first invoked */
 static BlindVals bvals =
 {
   30,
   3,
-  HORIZONTAL, /* orientation */
+  GIMP_ORIENTATION_HORIZONTAL,
   FALSE
 };
 
-MAIN ()
 
 static void
-query (void)
+blinds_class_init (BlindsClass *klass)
 {
-  static const GimpParamDef args[] =
-  {
-    { GIMP_PDB_INT32,    "run-mode",       "Interactive, non-interactive" },
-    { GIMP_PDB_IMAGE,    "image",          "Input image (unused)" },
-    { GIMP_PDB_DRAWABLE, "drawable",       "Input drawable" },
-    { GIMP_PDB_INT32,    "angle-dsp",      "Angle of Displacement " },
-    { GIMP_PDB_INT32,    "num-segments",   "Number of segments in blinds" },
-    { GIMP_PDB_INT32,    "orientation",    "orientation; 0 = Horizontal, 1 = Vertical" },
-    { GIMP_PDB_INT32,    "bg-transparent", "background transparent; FALSE,TRUE" }
-  };
+  GimpPlugInClass *plug_in_class = GIMP_PLUG_IN_CLASS (klass);
 
-  gimp_install_procedure (PLUG_IN_PROC,
-                          N_("Simulate an image painted on window blinds"),
-                          "More here later",
-                          "Andy Thomas",
-                          "Andy Thomas",
-                          "1997",
-                          N_("_Blinds..."),
-                          "RGB*, GRAY*",
-                          GIMP_PLUGIN,
-                          G_N_ELEMENTS (args), 0,
-                          args, NULL);
-
-  gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/Filters/Distorts");
+  plug_in_class->query_procedures = blinds_query_procedures;
+  plug_in_class->create_procedure = blinds_create_procedure;
+  plug_in_class->set_i18n         = STD_SET_I18N;
 }
 
 static void
-run (const gchar      *name,
-     gint              nparams,
-     const GimpParam  *param,
-     gint             *nreturn_vals,
-     GimpParam       **return_vals)
+blinds_init (Blinds *blinds)
+{
+}
+
+static GList *
+blinds_query_procedures (GimpPlugIn *plug_in)
+{
+  return g_list_append (NULL, g_strdup (PLUG_IN_PROC));
+}
+
+static GimpProcedure *
+blinds_create_procedure (GimpPlugIn  *plug_in,
+                         const gchar *name)
+{
+  GimpProcedure *procedure = NULL;
+
+  if (! strcmp (name, PLUG_IN_PROC))
+    {
+      procedure = gimp_image_procedure_new (plug_in, name,
+                                            GIMP_PDB_PROC_TYPE_PLUGIN,
+                                            blinds_run, NULL, NULL);
+
+      gimp_procedure_set_image_types (procedure, "RGB*, GRAY*");
+      gimp_procedure_set_sensitivity_mask (procedure,
+                                           GIMP_PROCEDURE_SENSITIVE_DRAWABLE);
+
+      gimp_procedure_set_menu_label (procedure, _("_Blinds..."));
+      gimp_procedure_add_menu_path (procedure, "<Image>/Filters/Distorts");
+
+      gimp_procedure_set_documentation (procedure,
+                                        _("Simulate an image painted on "
+                                          "window blinds"),
+                                        "More here later",
+                                        name);
+      gimp_procedure_set_attribution (procedure,
+                                      "Andy Thomas",
+                                      "Andy Thomas",
+                                      "1997");
+
+      GIMP_PROC_ARG_INT (procedure, "angle-displacement",
+                         "Angle displacement",
+                         "Angle of Displacement",
+                         0, 360, 30,
+                         G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_INT (procedure, "num-segments",
+                         "Num segments",
+                         "Number of segments in blinds",
+                         1, 1024, 3,
+                         G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_ENUM (procedure, "orientation",
+                          "Orientation",
+                          "The orientation",
+                          GIMP_TYPE_ORIENTATION_TYPE,
+                          GIMP_ORIENTATION_HORIZONTAL,
+                          G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_BOOLEAN (procedure, "bg-transparent",
+                             "BG transparent",
+                             "Background transparent",
+                             FALSE,
+                             G_PARAM_READWRITE);
+    }
+
+  return procedure;
+}
+
+static GimpValueArray *
+blinds_run (GimpProcedure        *procedure,
+            GimpRunMode           run_mode,
+            GimpImage            *image,
+            gint                  n_drawables,
+            GimpDrawable        **drawables,
+            const GimpValueArray *args,
+            gpointer              run_data)
 {
-  static GimpParam values[1];
   GimpDrawable *drawable;
-  GimpRunMode run_mode;
-  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
 
-  run_mode = param[0].data.d_int32;
+  gegl_init (NULL, NULL);
 
-  INIT_I18N ();
+  if (n_drawables != 1)
+    {
+      GError *error = NULL;
 
-  *nreturn_vals = 1;
-  *return_vals = values;
+      g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Procedure '%s' only works with one drawable."),
+                   PLUG_IN_PROC);
 
-  values[0].type = GIMP_PDB_STATUS;
-  values[0].data.d_status = status;
-
-  drawable = gimp_drawable_get (param[2].data.d_drawable);
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_CALLING_ERROR,
+                                               error);
+    }
+  else
+    {
+      drawable = drawables[0];
+    }
 
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
       gimp_get_data (PLUG_IN_PROC, &bvals);
-      if (! blinds_dialog(drawable))
-        {
-          gimp_drawable_detach (drawable);
-          return;
-        }
+
+      if (! blinds_dialog (drawable))
+        return gimp_procedure_new_return_values (procedure,
+                                                 GIMP_PDB_CANCEL,
+                                                 NULL);
       break;
 
     case GIMP_RUN_NONINTERACTIVE:
-      if (nparams != 7)
-        status = GIMP_PDB_CALLING_ERROR;
-      if (status == GIMP_PDB_SUCCESS)
-        {
-          bvals.angledsp = param[3].data.d_int32;
-          bvals.numsegs = param[4].data.d_int32;
-          bvals.orientation = param[5].data.d_int32;
-          bvals.bg_trans = param[6].data.d_int32;
-        }
+      bvals.angledsp    = GIMP_VALUES_GET_INT     (args, 0);
+      bvals.numsegs     = GIMP_VALUES_GET_INT     (args, 1);
+      bvals.orientation = GIMP_VALUES_GET_ENUM    (args, 2);
+      bvals.bg_trans    = GIMP_VALUES_GET_BOOLEAN (args, 3);
       break;
 
     case GIMP_RUN_WITH_LAST_VALS:
@@ -182,10 +253,10 @@ run (const gchar      *name,
       break;
     }
 
-  if (gimp_drawable_is_rgb (drawable->drawable_id) ||
-      gimp_drawable_is_gray (drawable->drawable_id))
+  if (gimp_drawable_is_rgb  (drawable) ||
+      gimp_drawable_is_gray (drawable))
     {
-      gimp_progress_init ( _("Adding blinds"));
+      gimp_progress_init (_("Adding blinds"));
 
       apply_blinds (drawable);
 
@@ -197,12 +268,12 @@ run (const gchar      *name,
     }
   else
     {
-      status = GIMP_PDB_EXECUTION_ERROR;
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_EXECUTION_ERROR,
+                                               NULL);
     }
 
-  values[0].data.d_status = status;
-
-  gimp_drawable_detach (drawable);
+  return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
 
@@ -214,37 +285,38 @@ blinds_dialog (GimpDrawable *drawable)
   GtkWidget *preview;
   GtkWidget *hbox;
   GtkWidget *frame;
-  GtkWidget *table;
-  GtkObject *size_data;
+  GtkWidget *grid;
+  GtkWidget *scale;
   GtkWidget *toggle;
   GtkWidget *horizontal;
   GtkWidget *vertical;
   gboolean   run;
 
-  gimp_ui_init (PLUG_IN_BINARY, TRUE);
+  gimp_ui_init (PLUG_IN_BINARY);
 
-  dialog = gimp_dialog_new (_("Blinds"), PLUG_IN_BINARY,
+  dialog = gimp_dialog_new (_("Blinds"), PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                            _("_Cancel"), GTK_RESPONSE_CANCEL,
+                            _("_OK"),     GTK_RESPONSE_OK,
 
                             NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
                                            GTK_RESPONSE_CANCEL,
                                            -1);
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
-  preview = gimp_aspect_preview_new (drawable, NULL);
+  preview = gimp_aspect_preview_new_from_drawable (drawable);
   gtk_box_pack_start (GTK_BOX (main_vbox), preview, TRUE, TRUE, 0);
   gtk_widget_show (preview);
 
@@ -252,17 +324,20 @@ blinds_dialog (GimpDrawable *drawable)
                             G_CALLBACK (dialog_update_preview),
                             drawable);
 
-  hbox = gtk_hbox_new (FALSE, 12);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   frame =
     gimp_int_radio_group_new (TRUE, _("Orientation"),
                               G_CALLBACK (gimp_radio_button_update),
-                              &bvals.orientation, bvals.orientation,
+                              &bvals.orientation, NULL, bvals.orientation,
 
-                              _("_Horizontal"), HORIZONTAL, &horizontal,
-                              _("_Vertical"),   VERTICAL,   &vertical,
+                              _("_Horizontal"), GIMP_ORIENTATION_HORIZONTAL,
+                              &horizontal,
+
+                              _("_Vertical"),   GIMP_ORIENTATION_VERTICAL,
+                              &vertical,
 
                               NULL);
   gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
@@ -292,41 +367,38 @@ blinds_dialog (GimpDrawable *drawable)
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), bvals.bg_trans);
 
-  if (!gimp_drawable_has_alpha (drawable->drawable_id))
+  if (! gimp_drawable_has_alpha (drawable))
     {
       gtk_widget_set_sensitive (toggle, FALSE);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), FALSE);
     }
 
-  table = gtk_table_new (2, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_box_pack_start (GTK_BOX (main_vbox), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
-  size_data = gimp_scale_entry_new (GTK_TABLE (table), 0, 0,
-                                    _("_Displacement:"), SCALE_WIDTH, 0,
-                                    bvals.angledsp, 1, 90, 1, 15, 0,
-                                    TRUE, 0, 0,
-                                    NULL, NULL);
-  g_signal_connect (size_data, "value-changed",
-                    G_CALLBACK (gimp_int_adjustment_update),
+  scale = gimp_scale_entry_new (_("_Displacement:"), bvals.angledsp, 1, 90, 0);
+  gimp_label_spin_set_increments (GIMP_LABEL_SPIN (scale), 1.0, 15.0);
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (blinds_scale_entry_update_int),
                     &bvals.angledsp);
-  g_signal_connect_swapped (size_data, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_box_pack_start (GTK_BOX (main_vbox), scale, FALSE, FALSE, 2);
+  gtk_widget_show (scale);
 
-  size_data = gimp_scale_entry_new (GTK_TABLE (table), 0, 1,
-                                    _("_Number of segments:"), SCALE_WIDTH, 0,
-                                    bvals.numsegs, 1, MAX_FANS, 1, 2, 0,
-                                    TRUE, 0, 0,
-                                    NULL, NULL);
-  g_signal_connect (size_data, "value-changed",
-                    G_CALLBACK (gimp_int_adjustment_update),
+  scale = gimp_scale_entry_new (_("_Number of segments:"), bvals.numsegs, 1, MAX_FANS, 0);
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (blinds_scale_entry_update_int),
                     &bvals.numsegs);
-  g_signal_connect_swapped (size_data, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_box_pack_start (GTK_BOX (main_vbox), scale, FALSE, FALSE, 2);
+  gtk_widget_show (scale);
 
   gtk_widget_show (dialog);
 
@@ -337,6 +409,13 @@ blinds_dialog (GimpDrawable *drawable)
   return run;
 }
 
+static void
+blinds_scale_entry_update_int (GimpLabelSpin *entry,
+                               gint          *value)
+{
+  *value = (gint) gimp_label_spin_get_value (entry);
+}
+
 static void
 blindsapply (guchar *srow,
              guchar *drow,
@@ -351,11 +430,11 @@ blindsapply (guchar *srow,
   gint     available;
 
   /* Make the row 'shrink' around points along its length */
-  /* The bvals.numsegs determins how many segments to slip it in to */
+  /* The bvals.numsegs determines how many segments to slip it in to */
   /* The angle is the conceptual 'rotation' of each of these segments */
 
   /* Note the row is considered to be made up of a two dim array actual
-   * pixel locations and the RGB colour at these locations.
+   * pixel locations and the RGB color at these locations.
    */
 
   /* In the process copy the src row to the destination row */
@@ -453,8 +532,7 @@ dialog_update_preview (GimpDrawable *drawable,
   gint     width, height, bpp;
 
   gimp_preview_get_size (preview, &width, &height);
-  bpp = gimp_drawable_bpp (drawable->drawable_id);
-  cache = gimp_drawable_get_thumbnail_data (drawable->drawable_id,
+  cache = gimp_drawable_get_thumbnail_data (drawable,
                                             &width, &height, &bpp);
   p = cache;
 
@@ -463,11 +541,19 @@ dialog_update_preview (GimpDrawable *drawable,
   if (bvals.bg_trans)
     gimp_rgb_set_alpha (&background, 0.0);
 
-  gimp_drawable_get_color_uchar (drawable->drawable_id, &background, bg);
+  if (gimp_drawable_is_gray (drawable))
+    {
+      bg[0] = gimp_rgb_luminance_uchar (&background);
+      gimp_rgba_get_uchar (&background, NULL, NULL, NULL, bg + 3);
+    }
+  else
+    {
+      gimp_rgba_get_uchar (&background, bg, bg + 1, bg + 2, bg + 3);
+    }
 
   buffer = g_new (guchar, width * height * bpp);
 
-  if (bvals.orientation)
+  if (bvals.orientation == GIMP_ORIENTATION_VERTICAL)
     {
       for (y = 0; y < height; y++)
         {
@@ -482,7 +568,7 @@ dialog_update_preview (GimpDrawable *drawable,
     {
       /* Horizontal blinds */
       /* Apply the blinds algo to a single column -
-       * this act as a transfomation matrix for the
+       * this act as a transformation matrix for the
        * rows. Make row 0 invalid so we can find it again!
        */
       gint i;
@@ -552,67 +638,71 @@ dialog_update_preview (GimpDrawable *drawable,
 static void
 apply_blinds (GimpDrawable *drawable)
 {
-  GimpPixelRgn  des_rgn;
-  GimpPixelRgn  src_rgn;
-  guchar       *src_rows, *des_rows;
-  gint          x, y;
-  GimpRGB       background;
-  guchar        bg[4];
-  gint          sel_x1, sel_y1, sel_x2, sel_y2;
-  gint          sel_width, sel_height;
+  GeglBuffer *src_buffer;
+  GeglBuffer *dest_buffer;
+  const Babl *format;
+  guchar     *src_rows, *des_rows;
+  gint        bytes;
+  gint        x, y;
+  GimpRGB     background;
+  guchar      bg[4];
+  gint        sel_x1, sel_y1;
+  gint        sel_width, sel_height;
 
   gimp_context_get_background (&background);
 
   if (bvals.bg_trans)
     gimp_rgb_set_alpha (&background, 0.0);
 
-  gimp_drawable_get_color_uchar (drawable->drawable_id, &background, bg);
+  gimp_rgba_get_uchar (&background, bg, bg + 1, bg + 2, bg + 3);
 
-  gimp_drawable_mask_bounds (drawable->drawable_id,
-                             &sel_x1, &sel_y1, &sel_x2, &sel_y2);
+  if (! gimp_drawable_mask_intersect (drawable,
+                                      &sel_x1, &sel_y1,
+                                      &sel_width, &sel_height))
+    return;
 
-  sel_width  = sel_x2 - sel_x1;
-  sel_height = sel_y2 - sel_y1;
+  if (gimp_drawable_has_alpha (drawable))
+    format = babl_format ("R'G'B'A u8");
+  else
+    format = babl_format ("R'G'B' u8");
 
-  gimp_pixel_rgn_init (&src_rgn, drawable,
-                       sel_x1, sel_y1, sel_width, sel_height, FALSE, FALSE);
-  gimp_pixel_rgn_init (&des_rgn, drawable,
-                       sel_x1, sel_y1, sel_width, sel_height, TRUE, TRUE);
+  bytes = babl_format_get_bytes_per_pixel (format);
 
-  src_rows = g_new (guchar, MAX (sel_width, sel_height) * 4 * STEP);
-  des_rows = g_new (guchar, MAX (sel_width, sel_height) * 4 * STEP);
+  src_buffer  = gimp_drawable_get_buffer (drawable);
+  dest_buffer = gimp_drawable_get_shadow_buffer (drawable);
 
-  if (bvals.orientation)
+  src_rows = g_new (guchar, MAX (sel_width, sel_height) * bytes * STEP);
+  des_rows = g_new (guchar, MAX (sel_width, sel_height) * bytes * STEP);
+
+  if (bvals.orientation == GIMP_ORIENTATION_VERTICAL)
     {
       for (y = 0; y < sel_height; y += STEP)
         {
           gint rr;
           gint step;
 
-          if((y + STEP) > sel_height)
+          if ((y + STEP) > sel_height)
             step = sel_height - y;
           else
             step = STEP;
 
-          gimp_pixel_rgn_get_rect (&src_rgn,
-                                   src_rows,
-                                   sel_x1,
-                                   sel_y1 + y,
-                                   sel_width,
-                                   step);
+          gegl_buffer_get (src_buffer,
+                           GEGL_RECTANGLE (sel_x1, sel_y1 + y,
+                                           sel_width, step), 1.0,
+                           format, src_rows,
+                           GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
           /* OK I could make this better */
           for (rr = 0; rr < STEP; rr++)
-            blindsapply (src_rows + (sel_width * rr * src_rgn.bpp),
-                         des_rows + (sel_width * rr * src_rgn.bpp),
-                         sel_width, src_rgn.bpp, bg);
+            blindsapply (src_rows + (sel_width * rr * bytes),
+                         des_rows + (sel_width * rr * bytes),
+                         sel_width, bytes, bg);
 
-          gimp_pixel_rgn_set_rect (&des_rgn,
-                                   des_rows,
-                                   sel_x1,
-                                   sel_y1 + y,
-                                   sel_width,
-                                   step);
+          gegl_buffer_set (dest_buffer,
+                           GEGL_RECTANGLE (sel_x1, sel_y1 + y,
+                                           sel_width, step), 0,
+                           format, des_rows,
+                           GEGL_AUTO_ROWSTRIDE);
 
           gimp_progress_update ((double) y / (double) sel_height);
         }
@@ -621,24 +711,24 @@ apply_blinds (GimpDrawable *drawable)
     {
       /* Horizontal blinds */
       /* Apply the blinds algo to a single column -
-       * this act as a transfomation matrix for the
+       * this act as a transformation matrix for the
        * rows. Make row 0 invalid so we can find it again!
        */
       gint    i;
-      gint   *sr  = g_new (gint, sel_height * 4);
-      gint   *dr  = g_new (gint, sel_height * 4);
-      guchar *dst = g_new (guchar, STEP * 4);
+      gint   *sr  = g_new (gint, sel_height * bytes);
+      gint   *dr  = g_new (gint, sel_height * bytes);
+      guchar *dst = g_new (guchar, STEP * bytes);
       guchar  dummybg[4];
 
       memset (dummybg, 0, 4);
-      memset (dr, 0, sel_height * 4); /* all dr rows are background rows */
+      memset (dr, 0, sel_height * bytes); /* all dr rows are background rows */
       for (y = 0; y < sel_height; y++)
         {
           sr[y] = y+1;
         }
 
       /* Hmmm. does this work portably? */
-      /* This "swaps the intergers around that are held in in the
+      /* This "swaps" the integers around that are held in in the
        * sr & dr arrays.
        */
       blindsapply ((guchar *) sr, (guchar *) dr,
@@ -649,9 +739,9 @@ apply_blinds (GimpDrawable *drawable)
         {
           int     j;
           guchar *bgdst;
-          bgdst = &dst[i * src_rgn.bpp];
+          bgdst = &dst[i * bytes];
 
-          for (j = 0 ; j < src_rgn.bpp; j++)
+          for (j = 0 ; j < bytes; j++)
             {
               bgdst[j] = bg[j];
             }
@@ -668,12 +758,11 @@ apply_blinds (GimpDrawable *drawable)
           else
             step = STEP;
 
-          gimp_pixel_rgn_get_rect (&src_rgn,
-                                   src_rows,
-                                   sel_x1 + x,
-                                   sel_y1,
-                                   step,
-                                   sel_height);
+          gegl_buffer_get (src_buffer,
+                           GEGL_RECTANGLE (sel_x1 + x, sel_y1,
+                                           step, sel_height), 1.0,
+                           format, src_rows,
+                           GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
           /* OK I could make this better */
           for (rr = 0; rr < sel_height; rr++)
@@ -686,18 +775,17 @@ apply_blinds (GimpDrawable *drawable)
               else
                 {
                   /* Draw line from src */
-                  p = src_rows + (step * src_rgn.bpp * (dr[rr] - 1));
+                  p = src_rows + (step * bytes * (dr[rr] - 1));
                 }
-              memcpy (des_rows + (rr * step * src_rgn.bpp), p,
-                      step * src_rgn.bpp);
+              memcpy (des_rows + (rr * step * bytes), p,
+                      step * bytes);
             }
 
-          gimp_pixel_rgn_set_rect (&des_rgn,
-                                   des_rows,
-                                   sel_x1 + x,
-                                   sel_y1,
-                                   step,
-                                   sel_height);
+          gegl_buffer_set (dest_buffer,
+                           GEGL_RECTANGLE (sel_x1 + x, sel_y1,
+                                           step, sel_height), 0,
+                           format, des_rows,
+                           GEGL_AUTO_ROWSTRIDE);
 
           gimp_progress_update ((double) x / (double) sel_width);
         }
@@ -710,8 +798,12 @@ apply_blinds (GimpDrawable *drawable)
   g_free (src_rows);
   g_free (des_rows);
 
-  gimp_drawable_flush (drawable);
-  gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
-  gimp_drawable_update (drawable->drawable_id,
+  g_object_unref (src_buffer);
+  g_object_unref (dest_buffer);
+
+  gimp_progress_update (1.0);
+
+  gimp_drawable_merge_shadow (drawable, TRUE);
+  gimp_drawable_update (drawable,
                         sel_x1, sel_y1, sel_width, sel_height);
 }
