@@ -83,17 +83,49 @@
 /* #define GIFDEBUG yesplease */
 
 
-/* Declare some local functions.
- */
-static void   query      (void);
-static void   run        (const gchar      *name,
-                          gint              nparams,
-                          const GimpParam  *param,
-                          gint             *nreturn_vals,
-                          GimpParam       **return_vals);
-static gint32 load_image (const gchar      *filename,
-                          gboolean          thumbnail,
-                          GError          **error);
+typedef struct _Gif      Gif;
+typedef struct _GifClass GifClass;
+
+struct _Gif
+{
+  GimpPlugIn      parent_instance;
+};
+
+struct _GifClass
+{
+  GimpPlugInClass parent_class;
+};
+
+
+#define GIF_TYPE  (gif_get_type ())
+#define GIF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIF_TYPE, Gif))
+
+GType                   gif_get_type         (void) G_GNUC_CONST;
+
+static GList          * gif_query_procedures (GimpPlugIn           *plug_in);
+static GimpProcedure  * gif_create_procedure (GimpPlugIn           *plug_in,
+                                              const gchar          *name);
+
+static GimpValueArray * gif_load             (GimpProcedure        *procedure,
+                                              GimpRunMode           run_mode,
+                                              GFile                *file,
+                                              const GimpValueArray *args,
+                                              gpointer              run_data);
+static GimpValueArray * gif_load_thumb       (GimpProcedure        *procedure,
+                                              GFile                *file,
+                                              gint                  size,
+                                              const GimpValueArray *args,
+                                              gpointer              run_data);
+
+static GimpImage      * load_image           (GFile                *file,
+                                              gboolean              thumbnail,
+                                              GError              **error);
+
+
+G_DEFINE_TYPE (Gif, gif, GIMP_TYPE_PLUG_IN)
+
+GIMP_MAIN (GIF_TYPE)
+DEFINE_STD_SET_I18N
 
 
 static guchar        used_cmap[3][256];
@@ -103,160 +135,171 @@ static guchar        gimp_cmap[768];
 static GimpParasite *comment_parasite = NULL;
 
 
-const GimpPlugInInfo PLUG_IN_INFO =
-{
-  NULL,  /* init_proc  */
-  NULL,  /* quit_proc  */
-  query, /* query_proc */
-  run,   /* run_proc   */
-};
-
-
-MAIN ()
-
 static void
-query (void)
+gif_class_init (GifClass *klass)
 {
-  static const GimpParamDef load_args[] =
-  {
-    { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
-    { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
-    { GIMP_PDB_STRING, "raw-filename", "The name entered"             }
-  };
-  static const GimpParamDef load_return_vals[] =
-  {
-    { GIMP_PDB_IMAGE,  "image",        "Output image"                 }
-  };
-  static const GimpParamDef thumb_args[] =
-  {
-    { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
-    { GIMP_PDB_INT32,  "thumb-size",   "Preferred thumbnail size"     }
-  };
-  static const GimpParamDef thumb_return_vals[] =
-  {
-    { GIMP_PDB_IMAGE,  "image",        "Output image"                 },
-    { GIMP_PDB_INT32,  "image-width",  "Width of full-sized image"    },
-    { GIMP_PDB_INT32,  "image-height", "Height of full-sized image"   }
-  };
+  GimpPlugInClass *plug_in_class = GIMP_PLUG_IN_CLASS (klass);
 
-  gimp_install_procedure (LOAD_PROC,
-                          "Loads files of Compuserve GIF file format",
-                          "FIXME: write help for gif_load",
-                          "Spencer Kimball, Peter Mattis, Adam Moss, David Koblas",
-                          "Spencer Kimball, Peter Mattis, Adam Moss, David Koblas",
-                          "1995-2006",
-                          N_("GIF image"),
-                          NULL,
-                          GIMP_PLUGIN,
-                          G_N_ELEMENTS (load_args),
-                          G_N_ELEMENTS (load_return_vals),
-                          load_args, load_return_vals);
-
-  gimp_register_file_handler_mime (LOAD_PROC, "image/gif");
-  gimp_register_magic_load_handler (LOAD_PROC,
-                                    "gif",
-                                    "",
-                                    "0,string,GIF8");
-
-  gimp_install_procedure (LOAD_THUMB_PROC,
-                          "Loads only the first frame of a GIF image, to be "
-                          "used as a thumbnail",
-                          "",
-                          "Sven Neumann",
-                          "Sven Neumann",
-                          "2006",
-                          NULL,
-                          NULL,
-                          GIMP_PLUGIN,
-                          G_N_ELEMENTS (thumb_args),
-                          G_N_ELEMENTS (thumb_return_vals),
-                          thumb_args, thumb_return_vals);
-
-  gimp_register_thumbnail_loader (LOAD_PROC, LOAD_THUMB_PROC);
+  plug_in_class->query_procedures = gif_query_procedures;
+  plug_in_class->create_procedure = gif_create_procedure;
+  plug_in_class->set_i18n         = STD_SET_I18N;
 }
 
 static void
-run (const gchar      *name,
-     gint              nparams,
-     const GimpParam  *param,
-     gint             *nreturn_vals,
-     GimpParam       **return_vals)
+gif_init (Gif *gif)
 {
-  static GimpParam   values[4];
-  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  GError            *error  = NULL;
-  gint32             image_ID;
+}
+
+static GList *
+gif_query_procedures (GimpPlugIn *plug_in)
+{
+  GList *list = NULL;
+
+  list = g_list_append (list, g_strdup (LOAD_THUMB_PROC));
+  list = g_list_append (list, g_strdup (LOAD_PROC));
+
+  return list;
+}
+
+static GimpProcedure *
+gif_create_procedure (GimpPlugIn  *plug_in,
+                      const gchar *name)
+{
+  GimpProcedure *procedure = NULL;
+
+  if (! strcmp (name, LOAD_PROC))
+    {
+      procedure = gimp_load_procedure_new (plug_in, name,
+                                           GIMP_PDB_PROC_TYPE_PLUGIN,
+                                           gif_load, NULL, NULL);
+
+      gimp_procedure_set_menu_label (procedure, _("GIF image"));
+
+      gimp_procedure_set_documentation (procedure,
+                                        "Loads files of Compuserve GIF "
+                                        "file format",
+                                        "FIXME: write help for gif_load",
+                                        name);
+      gimp_procedure_set_attribution (procedure,
+                                      "Spencer Kimball, Peter Mattis, "
+                                      "Adam Moss, David Koblas",
+                                      "Spencer Kimball, Peter Mattis, "
+                                      "Adam Moss, David Koblas",
+                                      "1995-2006");
+
+      gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
+                                          "image/gif");
+      gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
+                                          "gif");
+      gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
+                                      "0,string,GIF8");
+
+      gimp_load_procedure_set_thumbnail_loader (GIMP_LOAD_PROCEDURE (procedure),
+                                                LOAD_THUMB_PROC);
+    }
+  else if (! strcmp (name, LOAD_THUMB_PROC))
+    {
+      procedure = gimp_thumbnail_procedure_new (plug_in, name,
+                                                GIMP_PDB_PROC_TYPE_PLUGIN,
+                                                gif_load_thumb, NULL, NULL);
+
+      gimp_procedure_set_documentation (procedure,
+                                        "Loads only the first frame of a "
+                                        "GIF image, to be "
+                                        "used as a thumbnail",
+                                        "",
+                                        name);
+      gimp_procedure_set_attribution (procedure,
+                                      "Sven Neumann",
+                                      "Sven Neumann",
+                                      "2006");
+    }
+
+  return procedure;
+}
+
+
+static GimpValueArray *
+gif_load (GimpProcedure        *procedure,
+          GimpRunMode           run_mode,
+          GFile                *file,
+          const GimpValueArray *args,
+          gpointer              run_data)
+{
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  GError         *error = NULL;
 
-  INIT_I18N ();
   gegl_init (NULL, NULL);
 
-  *nreturn_vals = 1;
-  *return_vals  = values;
+  image = load_image (file, FALSE, &error);
 
-  values[0].type          = GIMP_PDB_STATUS;
-  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+  if (! image)
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
 
-  if (strcmp (name, LOAD_PROC) == 0)
-    {
-      image_ID = load_image (param[1].data.d_string, FALSE, &error);
-    }
-  else if (strcmp (name, LOAD_THUMB_PROC) == 0)
-    {
-      image_ID = load_image (param[0].data.d_string, TRUE, &error);
-    }
-  else
-    {
-      status = GIMP_PDB_CALLING_ERROR;
-    }
-
-  if (status == GIMP_PDB_SUCCESS)
-    {
-      if (image_ID != -1)
-        {
-          /* The GIF format only tells you how many bits per pixel
-           *  are in the image, not the actual number of used indices (D'OH!)
-           *
-           * So if we're not careful, repeated load/save of a transparent GIF
-           *  without intermediate indexed->RGB->indexed pumps up the number of
-           *  bits used, as we add an index each time for the transparent
-           *  color.  Ouch.  We either do some heavier analysis at save-time,
-           *  or trim down the number of GIMP colors at load-time.  We do the
-           *  latter for now.
-           */
+  /* The GIF format only tells you how many bits per pixel are in the
+   *  image, not the actual number of used indices (D'OH!)
+   *
+   * So if we're not careful, repeated load/save of a transparent GIF
+   *  without intermediate indexed->RGB->indexed pumps up the number
+   *  of bits used, as we add an index each time for the transparent
+   *  color.  Ouch.  We either do some heavier analysis at save-time,
+   *  or trim down the number of GIMP colors at load-time.  We do the
+   *  latter for now.
+   */
 #ifdef GIFDEBUG
-          g_print ("GIF: Highest used index is %d\n", highest_used_index);
+  g_print ("GIF: Highest used index is %d\n", highest_used_index);
 #endif
-          if (! promote_to_rgb)
-            gimp_image_set_colormap (image_ID,
-                                     gimp_cmap, highest_used_index + 1);
 
-          *nreturn_vals = 2;
-          values[1].type         = GIMP_PDB_IMAGE;
-          values[1].data.d_image = image_ID;
+  if (! promote_to_rgb)
+    gimp_image_set_colormap (image,
+                             gimp_cmap, highest_used_index + 1);
 
-          if (strcmp (name, LOAD_THUMB_PROC) == 0)
-            {
-              *nreturn_vals = 4;
-              values[2].type         = GIMP_PDB_INT32;
-              values[2].data.d_int32 = gimp_image_width (image_ID);
-              values[3].type         = GIMP_PDB_INT32;
-              values[3].data.d_int32 = gimp_image_height (image_ID);
-            }
-        }
-      else
-        {
-          status = GIMP_PDB_EXECUTION_ERROR;
+  return_vals = gimp_procedure_new_return_values (procedure,
+                                                  GIMP_PDB_SUCCESS,
+                                                  NULL);
 
-          if (error)
-            {
-              *nreturn_vals = 2;
-              values[1].type          = GIMP_PDB_STRING;
-              values[1].data.d_string = error->message;
-            }
-        }
-    }
+  GIMP_VALUES_SET_IMAGE (return_vals, 1, image);
 
-  values[0].data.d_status = status;
+  return return_vals;
+}
+
+static GimpValueArray *
+gif_load_thumb (GimpProcedure        *procedure,
+                GFile                *file,
+                gint                  size,
+                const GimpValueArray *args,
+                gpointer              run_data)
+{
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  GError         *error = NULL;
+
+  gegl_init (NULL, NULL);
+
+  image = load_image (file, TRUE, &error);
+
+  if (! image)
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
+
+  if (! promote_to_rgb)
+    gimp_image_set_colormap (image, gimp_cmap, highest_used_index + 1);
+
+  return_vals = gimp_procedure_new_return_values (procedure,
+                                                  GIMP_PDB_SUCCESS,
+                                                  NULL);
+
+  GIMP_VALUES_SET_IMAGE (return_vals, 1, image);
+  GIMP_VALUES_SET_INT   (return_vals, 2, gimp_image_get_width  (image));
+  GIMP_VALUES_SET_INT   (return_vals, 3, gimp_image_get_height (image));
+
+  gimp_value_array_truncate (return_vals, 4);
+
+  return return_vals;
 }
 
 
@@ -318,7 +361,7 @@ static gint     LZWReadByte  (FILE        *fd,
                               gint         just_reset_LZW,
                               gint         input_code_size);
 static gboolean ReadImage    (FILE        *fd,
-                              const gchar *filename,
+                              GFile       *file,
                               gint         len,
                               gint         height,
                               CMap         cmap,
@@ -330,65 +373,65 @@ static gboolean ReadImage    (FILE        *fd,
                               guint        toppos,
                               guint        screenwidth,
                               guint        screenheight,
-                              gint32      *image_ID);
+                              GimpImage  **image);
 
 
-static gint32
-load_image (const gchar  *filename,
-            gboolean      thumbnail,
-            GError      **error)
+static GimpImage *
+load_image (GFile     *file,
+            gboolean   thumbnail,
+            GError   **error)
 {
-  FILE     *fd;
-  guchar    buf[16];
-  guchar    c;
-  CMap      localColorMap;
-  gint      grayScale;
-  gboolean  useGlobalColormap;
-  gint      bitPixel;
-  gint      imageCount = 0;
-  gchar     version[4];
-  gint32    image_ID = -1;
-  gboolean  status;
+  FILE      *fd;
+  guchar     buf[16];
+  guchar     c;
+  CMap       localColorMap;
+  gint       grayScale;
+  gboolean   useGlobalColormap;
+  gint       bitPixel;
+  gint       imageCount = 0;
+  GimpImage *image      = NULL;
+  gboolean   status;
 
   gimp_progress_init_printf (_("Opening '%s'"),
-                             gimp_filename_to_utf8 (filename));
+                             gimp_file_get_utf8_name (file));
 
-  fd = g_fopen (filename, "rb");
+  fd = g_fopen (g_file_peek_path (file), "rb");
 
   if (! fd)
     {
       g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
                    _("Could not open '%s' for reading: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
-      return -1;
+                   gimp_file_get_utf8_name (file), g_strerror (errno));
+      return NULL;
     }
 
   if (! ReadOK (fd, buf, 6))
     {
       g_message ("Error reading magic number");
-      return -1;
+      fclose (fd);
+      return NULL;
     }
 
   if (strncmp ((gchar *) buf, "GIF", 3) != 0)
     {
       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
                    "%s", _("This is not a GIF file"));
-      return -1;
+      fclose (fd);
+      return NULL;
     }
 
-  strncpy (version, (gchar *) buf + 3, 3);
-  version[3] = '\0';
-
-  if ((strcmp (version, "87a") != 0) && (strcmp (version, "89a") != 0))
+  if ((strncmp ((gchar *) buf + 3, "87a", 3) != 0) && (strncmp ((gchar *) buf + 3, "89a", 3) != 0))
     {
       g_message ("Bad version number, not '87a' or '89a'");
-      return -1;
+      fclose (fd);
+      return NULL;
     }
 
   if (! ReadOK (fd, buf, 7))
     {
       g_message ("Failed to read screen descriptor");
-      return -1;
+      fclose (fd);
+      return NULL;
     }
 
   GifScreen.Width           = LM_to_uint (buf[0], buf[1]);
@@ -405,7 +448,8 @@ load_image (const gchar  *filename,
                           &GifScreen.GrayScale))
         {
           g_message ("Error reading global colormap");
-          return -1;
+          fclose (fd);
+          return NULL;
         }
     }
 
@@ -422,13 +466,15 @@ load_image (const gchar  *filename,
       if (! ReadOK (fd, &c, 1))
         {
           g_message ("EOF / read error on image data");
-          return image_ID; /* will be -1 if failed on first image! */
+          fclose (fd);
+          return image; /* will be NULL if failed on first image! */
         }
 
       if (c == ';')
         {
           /* GIF terminator */
-          return image_ID;
+          fclose (fd);
+          return image;
         }
 
       if (c == '!')
@@ -437,7 +483,8 @@ load_image (const gchar  *filename,
           if (! ReadOK (fd, &c, 1))
             {
               g_message ("EOF / read error on extension function code");
-              return image_ID; /* will be -1 if failed on first image! */
+              fclose (fd);
+              return image; /* will be NULL if failed on first image! */
             }
 
           DoExtension (fd, c);
@@ -456,7 +503,8 @@ load_image (const gchar  *filename,
       if (! ReadOK (fd, buf, 9))
         {
           g_message ("Couldn't read left/top/width/height");
-          return image_ID; /* will be -1 if failed on first image! */
+          fclose (fd);
+          return image; /* will be NULL if failed on first image! */
         }
 
       useGlobalColormap = !BitSet (buf[8], LOCALCOLORMAP);
@@ -468,10 +516,11 @@ load_image (const gchar  *filename,
           if (! ReadColorMap (fd, bitPixel, localColorMap, &grayScale))
             {
               g_message ("Error reading local colormap");
-              return image_ID; /* will be -1 if failed on first image! */
+              fclose (fd);
+              return image; /* will be NULL if failed on first image! */
             }
 
-          status = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
+          status = ReadImage (fd, file, LM_to_uint (buf[4], buf[5]),
                               LM_to_uint (buf[6], buf[7]),
                               localColorMap, bitPixel,
                               grayScale,
@@ -480,11 +529,11 @@ load_image (const gchar  *filename,
                               (guint) LM_to_uint (buf[2], buf[3]),
                               GifScreen.Width,
                               GifScreen.Height,
-                              &image_ID);
+                              &image);
         }
       else
         {
-          status = ReadImage (fd, filename, LM_to_uint (buf[4], buf[5]),
+          status = ReadImage (fd, file, LM_to_uint (buf[4], buf[5]),
                               LM_to_uint (buf[6], buf[7]),
                               GifScreen.ColorMap, GifScreen.BitPixel,
                               GifScreen.GrayScale,
@@ -493,7 +542,7 @@ load_image (const gchar  *filename,
                               (guint) LM_to_uint (buf[2], buf[3]),
                               GifScreen.Width,
                               GifScreen.Height,
-                              &image_ID);
+                              &image);
         }
 
       if (!status)
@@ -504,7 +553,7 @@ load_image (const gchar  *filename,
       if (comment_parasite != NULL)
         {
           if (! thumbnail)
-            gimp_image_attach_parasite (image_ID, comment_parasite);
+            gimp_image_attach_parasite (image, comment_parasite);
 
           gimp_parasite_free (comment_parasite);
           comment_parasite = NULL;
@@ -515,7 +564,9 @@ load_image (const gchar  *filename,
         break;
     }
 
-  return image_ID;
+  fclose (fd);
+
+  return image;
 }
 
 static gboolean
@@ -860,6 +911,11 @@ LZWReadByte (FILE *fd,
 
       while (code >= clear_code && sp < &(stack[STACK_SIZE]))
         {
+          if (code >= (1 << MAX_LZW_BITS))
+            {
+              g_message ("Invalid table entry.  Corrupt file.");
+              return -1;
+            }
           *sp++ = table[1][code];
           if (code == table[0][code])
             {
@@ -901,7 +957,7 @@ LZWReadByte (FILE *fd,
 
 static gboolean
 ReadImage (FILE        *fd,
-           const gchar *filename,
+           GFile       *file,
            gint         len,
            gint         height,
            CMap         cmap,
@@ -913,11 +969,11 @@ ReadImage (FILE        *fd,
            guint        toppos,
            guint        screenwidth,
            guint        screenheight,
-           gint32      *image_ID)
+           GimpImage  **image)
 {
   static gint   frame_number = 1;
 
-  gint32        layer_ID;
+  GimpLayer    *layer;
   GeglBuffer   *buffer;
   guchar       *dest, *temp;
   guchar        c;
@@ -934,7 +990,7 @@ ReadImage (FILE        *fd,
   if (len < 1 || height < 1)
     {
       g_message ("Bogus frame dimensions");
-      *image_ID = -1;
+      *image = NULL;
       return FALSE;
     }
 
@@ -944,14 +1000,14 @@ ReadImage (FILE        *fd,
   if (! ReadOK (fd, &c, 1))
     {
       g_message ("EOF / read error on image data");
-      *image_ID = -1;
+      *image = NULL;
       return FALSE;
     }
 
   if (LZWReadByte (fd, TRUE, c) < 0)
     {
       g_message ("Error while reading");
-      *image_ID = -1;
+      *image = NULL;
       return FALSE;
     }
 
@@ -964,8 +1020,8 @@ ReadImage (FILE        *fd,
       if (screenheight == 0)
         screenheight = height;
 
-      *image_ID = gimp_image_new (screenwidth, screenheight, GIMP_INDEXED);
-      gimp_image_set_filename (*image_ID, filename);
+      *image = gimp_image_new (screenwidth, screenheight, GIMP_INDEXED);
+      gimp_image_set_file (*image, file);
 
       for (i = 0, j = 0; i < ncols; i++)
         {
@@ -974,7 +1030,7 @@ ReadImage (FILE        *fd,
           used_cmap[2][i] = gimp_cmap[j++] = cmap[2][i];
         }
 
-      gimp_image_set_colormap (*image_ID, gimp_cmap, ncols);
+      gimp_image_set_colormap (*image, gimp_cmap, ncols);
 
       if (Gif89.delayTime < 0)
         framename = g_strdup (_("Background"));
@@ -986,17 +1042,19 @@ ReadImage (FILE        *fd,
 
       if (Gif89.transparent == -1)
         {
-          layer_ID = gimp_layer_new (*image_ID, framename,
-                                     len, height,
-                                     GIMP_INDEXED_IMAGE,
-                                     100, GIMP_LAYER_MODE_NORMAL_LEGACY);
+          layer = gimp_layer_new (*image, framename,
+                                  len, height,
+                                  GIMP_INDEXED_IMAGE,
+                                  100,
+                                  gimp_image_get_default_new_layer_mode (*image));
         }
       else
         {
-          layer_ID = gimp_layer_new (*image_ID, framename,
-                                     len, height,
-                                     GIMP_INDEXEDA_IMAGE,
-                                     100, GIMP_LAYER_MODE_NORMAL_LEGACY);
+          layer = gimp_layer_new (*image, framename,
+                                  len, height,
+                                  GIMP_INDEXEDA_IMAGE,
+                                  100,
+                                  gimp_image_get_default_new_layer_mode (*image));
           alpha_frame=TRUE;
         }
 
@@ -1005,7 +1063,7 @@ ReadImage (FILE        *fd,
   else /* NOT FIRST FRAME */
     {
       gimp_progress_set_text_printf (_("Opening '%s' (frame %d)"),
-                                     gimp_filename_to_utf8 (filename),
+                                     gimp_file_get_utf8_name (file),
                                      frame_number);
       gimp_progress_pulse ();
 
@@ -1025,7 +1083,7 @@ ReadImage (FILE        *fd,
 #ifdef GIFDEBUG
                   g_print ("GIF: Promoting image to RGB...\n");
 #endif
-                  gimp_image_convert_rgb (*image_ID);
+                  gimp_image_convert_rgb (*image);
 
                   break;
                 }
@@ -1077,30 +1135,32 @@ ReadImage (FILE        *fd,
         }
       previous_disposal = Gif89.disposal;
 
-      layer_ID = gimp_layer_new (*image_ID, framename,
-                                 len, height,
-                                 promote_to_rgb ?
-                                 GIMP_RGBA_IMAGE : GIMP_INDEXEDA_IMAGE,
-                                 100, GIMP_LAYER_MODE_NORMAL_LEGACY);
+      layer = gimp_layer_new (*image, framename,
+                              len, height,
+                              promote_to_rgb ?
+                              GIMP_RGBA_IMAGE : GIMP_INDEXEDA_IMAGE,
+                              100,
+                              gimp_image_get_default_new_layer_mode (*image));
       alpha_frame = TRUE;
       g_free (framename);
     }
 
   frame_number++;
 
-  gimp_image_insert_layer (*image_ID, layer_ID, -1, 0);
-  gimp_layer_translate (layer_ID, (gint) leftpos, (gint) toppos);
+  gimp_image_insert_layer (*image, layer, NULL, 0);
+  gimp_item_transform_translate (GIMP_ITEM (layer), (gint) leftpos, (gint) toppos);
 
   cur_progress = 0;
   max_progress = height;
 
   if (len > (G_MAXSIZE / height / (alpha_frame ? (promote_to_rgb ? 4 : 2) : 1)))
-  {
-    g_message ("'%s' has a larger image size than GIMP can handle.",
-               gimp_filename_to_utf8 (filename));
-    *image_ID = -1;
-    return FALSE;
-  }
+    {
+      g_message ("'%s' has a larger image size than GIMP can handle.",
+                 gimp_file_get_utf8_name (file));
+      gimp_image_delete (*image);
+      *image = NULL;
+      return FALSE;
+    }
 
   if (alpha_frame)
     dest = (guchar *) g_malloc ((gsize)len * (gsize)height * (promote_to_rgb ? 4 : 2));
@@ -1117,7 +1177,8 @@ ReadImage (FILE        *fd,
       /* I don't see how one would easily construct a GIF in which
          this could happen, but it's a mad mad world. */
       g_message ("Ouch!  Can't handle non-alpha RGB frames.\n"
-                 "Please file a bug report in GIMP's bugzilla.");
+                 "Please file a bug report at "
+                 "https://gitlab.gnome.org/GNOME/gimp/issues");
       gimp_quit ();
     }
 
@@ -1215,7 +1276,7 @@ ReadImage (FILE        *fd,
     }
 
  fini:
-  buffer = gimp_drawable_get_buffer (layer_ID);
+  buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (layer));
 
   gegl_buffer_set (buffer, GEGL_RECTANGLE (0, 0, len, height), 0,
                    NULL, dest, GEGL_AUTO_ROWSTRIDE);
